@@ -135,6 +135,8 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
   private GobblinHelixJobScheduler jobScheduler;
   @Getter
   private JobConfigurationManager jobConfigurationManager;
+  @Getter
+  private volatile boolean started = false;
 
   protected final String clusterName;
   @Getter
@@ -317,6 +319,7 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
     } else {
       startAppLauncherAndServices();
     }
+    this.started = true;
   }
 
   /**
@@ -379,10 +382,6 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
    * Build the {@link JobConfigurationManager} for the Application Master.
    */
   private JobConfigurationManager buildJobConfigurationManager(Config config) {
-    return create(config);
-  }
-
-  private JobConfigurationManager create(Config config) {
     try {
       List<Object> argumentList = (this.jobCatalog != null)? ImmutableList.of(this.eventBus, config, this.jobCatalog, this.fs) :
           ImmutableList.of(this.eventBus, config, this.fs);
