@@ -176,8 +176,10 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
     this.taskStateCollectorService = new TaskStateCollectorService(jobProps,
         this.jobContext.getJobState(),
         this.eventBus,
+        this.eventSubmitter,
         this.stateStores.getTaskStateStore(),
-        this.outputTaskStateDir);
+        this.outputTaskStateDir,
+        this.getIssueRepository());
 
     this.helixMetrics = helixMetrics;
     startCancellationExecutor();
