@@ -76,6 +76,12 @@ public class TimeBasedSubDirDatasetsFinder extends DatasetsFinder {
   public static final String COMPACTION_TIMEBASED_MIN_TIME_AGO = COMPACTION_TIMEBASED_PREFIX + "min.time.ago";
   public static final String DEFAULT_COMPACTION_TIMEBASED_MIN_TIME_AGO = "1d";
 
+  // The latest compaction run time to be processed. Format = ?m?d?h.
+  public static final String MIN_RECOMPACTION_DURATION =
+      COMPACTION_TIMEBASED_PREFIX + "min.recompaction.duration";
+  // By default we don't apply this limitation
+  public static final String DEFAULT_MIN_RECOMPACTION_DURATION = "0h";
+
   protected final String folderTimePattern;
   protected final String subDirPattern;
   protected final DateTimeZone timeZone;
