@@ -75,10 +75,19 @@ public class OrcUtils {
 
   public static TypeDescription getTypeDescriptionFromFile(Configuration conf, Path orcFilePath)
       throws IOException {
-    return getRecordReaderFromFile(conf, orcFilePath).getSchema();
+    return getFileReader(conf, orcFilePath).getSchema();
   }
 
-  public static Reader getRecordReaderFromFile(Configuration conf, Path orcFilePath)
+  /**
+   * @deprecated Since the method name isn't accurate. Please calling {@link this#getFileReader(Configuration, Path)}
+   * directly
+   */
+  @Deprecated
+  public static Reader getRecordReaderFromFile(Configuration conf, Path orcFilePath) throws IOException {
+    return getFileReader(conf, orcFilePath);
+  }
+
+  public static Reader getFileReader(Configuration conf, Path orcFilePath)
       throws IOException {
     return OrcFile.createReader(orcFilePath, new OrcFile.ReaderOptions(conf));
   }
