@@ -17,10 +17,8 @@
 package org.apache.gobblin.writer;
 
 import java.io.IOException;
-import java.util.Map;
 
 import org.apache.gobblin.annotation.Alpha;
-import org.apache.gobblin.source.extractor.CheckpointableWatermark;
 import org.apache.gobblin.stream.RecordEnvelope;
 
 
