@@ -18,13 +18,9 @@
 package org.apache.gobblin.writer;
 
 import java.io.IOException;
-import java.util.Map;
 
 import lombok.extern.slf4j.Slf4j;
 
-import org.apache.gobblin.source.extractor.CheckpointableWatermark;
-import org.apache.gobblin.stream.RecordEnvelope;
-
 
 /**
  * A simple console writer that prints the record to stdout
