@@ -47,6 +47,8 @@ import org.apache.gobblin.data.management.copy.hive.HiveDatasetFinder;
 import org.apache.gobblin.hive.HiveMetastoreClientPool;
 import org.apache.gobblin.util.ConfigUtils;
 
+import static org.apache.gobblin.data.management.conversion.hive.task.HiveConverterUtils.getOutputDataLocation;
+
 
 /**
  * <p>
@@ -122,7 +124,7 @@ public class ConvertibleHiveDataset extends HiveDataset {
 
     for (String format : this.destFormats) {
       if (this.datasetConfig.hasPath(format)) {
-        log.debug("Found desination format: " + format);
+        log.debug("Found destination format: " + format);
         this.destConversionConfigs.put(format, new ConversionConfig(this.datasetConfig.getConfig(format), table, format));
 
       }
@@ -141,7 +143,9 @@ public class ConvertibleHiveDataset extends HiveDataset {
       String destTable = conversionConfigForFormat.get().getDestinationDbName() + "." + conversionConfigForFormat.get()
           .getDestinationTableName();
       DatasetDescriptor dest = new DatasetDescriptor(DatasetConstants.PLATFORM_HIVE, destTable);
-      String destLocation = conversionConfigForFormat.get().getDestinationDataPath() + Path.SEPARATOR + "final";
+      String destLocation = conversionConfigForFormat.get().getDataDstPathUseSubdir()
+          ? getOutputDataLocation(conversionConfigForFormat.get().getDestinationDataPath())
+          : conversionConfigForFormat.get().getDestinationDataPath();
       dest.addMetadata(DatasetConstants.FS_SCHEME, getSourceDataset().getMetadata().get(DatasetConstants.FS_SCHEME));
       dest.addMetadata(DatasetConstants.FS_LOCATION, destLocation);
       destDatasets.add(dest);
@@ -154,6 +158,7 @@ public class ConvertibleHiveDataset extends HiveDataset {
       String sourceTable = getTable().getDbName() + "." + getTable().getTableName();
       DatasetDescriptor source = new DatasetDescriptor(DatasetConstants.PLATFORM_HIVE, sourceTable);
       Path sourcePath = getTable().getDataLocation();
+      log.info(String.format("[%s]Source path %s being used in conversion", this.getClass().getName(), sourcePath));
       String sourceLocation = Path.getPathWithoutSchemeAndAuthority(sourcePath).toString();
       FileSystem sourceFs = sourcePath.getFileSystem(new Configuration());
       source.addMetadata(DatasetConstants.FS_SCHEME, sourceFs.getScheme());
