@@ -17,15 +17,6 @@
 
 package org.apache.gobblin.data.management.copy;
 
-import org.apache.gobblin.commit.CommitStep;
-import org.apache.gobblin.data.management.copy.entities.PrePublishStep;
-import org.apache.gobblin.data.management.dataset.DatasetUtils;
-import org.apache.gobblin.dataset.FileSystemDataset;
-import org.apache.gobblin.util.PathUtils;
-import org.apache.gobblin.util.FileListUtils;
-import org.apache.gobblin.util.commit.DeleteFileCommitStep;
-
-import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.util.Collection;
 import java.util.List;
@@ -42,11 +33,22 @@ import com.google.common.base.Optional;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 
+import lombok.extern.slf4j.Slf4j;
+
+import org.apache.gobblin.commit.CommitStep;
+import org.apache.gobblin.data.management.copy.entities.PrePublishStep;
+import org.apache.gobblin.data.management.dataset.DatasetUtils;
+import org.apache.gobblin.dataset.FileSystemDataset;
+import org.apache.gobblin.util.FileListUtils;
+import org.apache.gobblin.util.PathUtils;
+import org.apache.gobblin.util.commit.DeleteFileCommitStep;
+
 
 /**
  * Implementation of {@link CopyableDataset} that creates a {@link CopyableFile} for every file that is a descendant if
  * the root directory.
  */
+@Slf4j
 public class RecursiveCopyableDataset implements CopyableDataset, FileSystemDataset {
 
   private static final String CONFIG_PREFIX = CopyConfiguration.COPY_PREFIX + ".recursive";
@@ -179,7 +181,8 @@ public class RecursiveCopyableDataset implements CopyableDataset, FileSystemData
     try {
       return FileListUtils
           .listFilesToCopyAtPath(fs, path, fileFilter, applyFilterToDirectories, includeEmptyDirectories);
-    } catch (FileNotFoundException fnfe) {
+    } catch (IOException e) {
+      log.info(String.format("Could not find any files on target path due to %s. Returning an empty list of files.", e.getClass().getCanonicalName()));
       return Lists.newArrayList();
     }
   }
