@@ -27,7 +27,6 @@ import java.util.concurrent.Callable;
 import java.util.concurrent.TimeUnit;
 
 import org.apache.commons.lang.exception.ExceptionUtils;
-import org.apache.gobblin.metrics.reporter.FileFailureEventReporter;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
@@ -48,6 +47,8 @@ import com.google.common.collect.Lists;
 import com.google.common.io.Closer;
 import com.typesafe.config.Config;
 
+import lombok.Getter;
+
 import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.gobblin.configuration.State;
 import org.apache.gobblin.metrics.graphite.GraphiteConnectionType;
@@ -56,6 +57,7 @@ import org.apache.gobblin.metrics.graphite.GraphiteReporter;
 import org.apache.gobblin.metrics.influxdb.InfluxDBConnectionType;
 import org.apache.gobblin.metrics.influxdb.InfluxDBEventReporter;
 import org.apache.gobblin.metrics.influxdb.InfluxDBReporter;
+import org.apache.gobblin.metrics.reporter.FileFailureEventReporter;
 import org.apache.gobblin.metrics.reporter.OutputStreamEventReporter;
 import org.apache.gobblin.metrics.reporter.OutputStreamReporter;
 import org.apache.gobblin.metrics.reporter.ScheduledReporter;
@@ -70,8 +72,10 @@ import org.apache.gobblin.util.PropertiesUtils;
  */
 public class GobblinMetrics {
 
+  public static final String METRICS_ID_PREFIX = "gobblin.metrics.";
   public static final String METRICS_STATE_CUSTOM_TAGS = "metrics.state.custom.tags";
 
+  @Getter
   protected static final GobblinMetricsRegistry GOBBLIN_METRICS_REGISTRY = GobblinMetricsRegistry.getInstance();
 
   /**
@@ -147,7 +151,7 @@ public class GobblinMetrics {
    * @return a {@link GobblinMetrics} instance
    */
   public static GobblinMetrics get(final String id, final MetricContext parentContext, final List<Tag<?>> tags) {
-    return GOBBLIN_METRICS_REGISTRY.getOrDefault(id, new Callable<GobblinMetrics>() {
+    return GOBBLIN_METRICS_REGISTRY.getOrCreate(id, new Callable<GobblinMetrics>() {
       @Override
       public GobblinMetrics call()
           throws Exception {
@@ -350,7 +354,8 @@ public class GobblinMetrics {
    * Starts metric reporting and appends the given metrics file suffix to the current value of
    * {@link ConfigurationKeys#METRICS_FILE_SUFFIX}.
    */
-  public void startMetricReportingWithFileSuffix(State state, String metricsFileSuffix) {
+  public void startMetricReportingWithFileSuffix(State state, String metricsFileSuffix)
+      throws MultiReporterException {
     Properties metricsReportingProps = new Properties();
     metricsReportingProps.putAll(state.getProperties());
 
@@ -370,7 +375,8 @@ public class GobblinMetrics {
    *
    * @param configuration configuration properties
    */
-  public void startMetricReporting(Configuration configuration) {
+  public void startMetricReporting(Configuration configuration)
+      throws MultiReporterException {
     Properties props = new Properties();
     for (Map.Entry<String, String> entry : configuration) {
       props.put(entry.getKey(), entry.getValue());
@@ -383,7 +389,7 @@ public class GobblinMetrics {
    *
    * @param properties configuration properties
    */
-  public void startMetricReporting(Properties properties) {
+  public void startMetricReporting(Properties properties) throws MultiReporterException {
     if (this.metricsReportingStarted) {
       LOGGER.warn("Metric reporting has already started");
       return;
@@ -396,21 +402,20 @@ public class GobblinMetrics {
 
     long startTime = System.currentTimeMillis();
 
-    try {
-      // Build and start the JMX reporter
-      buildJmxMetricReporter(properties);
-      if (this.jmxReporter.isPresent()) {
-        LOGGER.info("Will start reporting metrics to JMX");
-        this.jmxReporter.get().start();
-      }
+    List<MetricReporterException> reporterExceptions = Lists.newArrayList();
 
-      // Build all other reporters
-      buildFileMetricReporter(properties);
-      buildKafkaMetricReporter(properties);
-      buildGraphiteMetricReporter(properties);
-      buildInfluxDBMetricReporter(properties);
-      buildCustomMetricReporters(properties);
-      buildFileFailureEventReporter(properties);
+    try {
+      for (ReporterSinkType sinkType: ReporterSinkType.values()) {
+        if (sinkType.equals(ReporterSinkType.CUSTOM)) {
+          buildCustomMetricReporters(properties, reporterExceptions);
+        } else {
+          try {
+            buildReporter(properties, sinkType);
+          } catch (MultiReporterException e) {
+            reporterExceptions.addAll(e.getExceptions());
+          }
+        }
+      }
 
       // Start reporters that implement org.apache.gobblin.metrics.report.ScheduledReporter
       RootMetricContext.get().startReporting();
@@ -427,6 +432,36 @@ public class GobblinMetrics {
     this.metricsReportingStarted = true;
     LOGGER.info("Metrics reporting has been started in {} ms: GobblinMetrics {}",
         System.currentTimeMillis() - startTime, this.toString());
+
+    if (!reporterExceptions.isEmpty()) {
+      throw new MultiReporterException("Could not create one or more reporters", reporterExceptions);
+    }
+  }
+
+  private void buildReporter(Properties properties, ReporterSinkType sinkType) throws MultiReporterException {
+    switch (sinkType) {
+      case JMX:
+        buildJmxMetricReporter(properties);
+        break;
+      case FILE:
+        buildFileMetricReporter(properties);
+        break;
+      case KAFKA:
+        buildKafkaMetricReporter(properties);
+        break;
+      case GRAPHITE:
+        buildGraphiteMetricReporter(properties);
+        break;
+      case INFLUXDB:
+        buildInfluxDBMetricReporter(properties);
+        break;
+      case FILE_FAILURE:
+        buildFileFailureEventReporter(properties);
+        break;
+      default:
+        LOGGER.error("Unknown reporter sink type: {}", sinkType.name());
+        break;
+    }
   }
 
   /**
@@ -469,7 +504,8 @@ public class GobblinMetrics {
     LOGGER.info("Metrics reporting stopped successfully");
   }
 
-  private void buildFileMetricReporter(Properties properties) {
+  private void buildFileMetricReporter(Properties properties)
+      throws MultiReporterException {
     if (!Boolean.valueOf(properties.getProperty(ConfigurationKeys.METRICS_REPORTING_FILE_ENABLED_KEY,
         ConfigurationKeys.DEFAULT_METRICS_REPORTING_FILE_ENABLED))) {
       return;
@@ -477,9 +513,9 @@ public class GobblinMetrics {
     LOGGER.info("Reporting metrics to log files");
 
     if (!properties.containsKey(ConfigurationKeys.METRICS_LOG_DIR_KEY)) {
-      LOGGER.error(
-          "Not reporting metrics to log files because " + ConfigurationKeys.METRICS_LOG_DIR_KEY + " is undefined");
-      return;
+      MetricReporterException e = new MetricReporterException(
+          "Not reporting metrics to log files because " + ConfigurationKeys.METRICS_LOG_DIR_KEY + " is undefined", ReporterType.METRIC, ReporterSinkType.FILE);
+      throw new MultiReporterException("Failed to create file metric reporter", Lists.newArrayList(e));
     }
 
     try {
@@ -489,8 +525,7 @@ public class GobblinMetrics {
       // Each job gets its own metric log subdirectory
       Path metricsLogDir = new Path(properties.getProperty(ConfigurationKeys.METRICS_LOG_DIR_KEY), this.getName());
       if (!fs.exists(metricsLogDir) && !fs.mkdirs(metricsLogDir)) {
-        LOGGER.error("Failed to create metric log directory for metrics " + this.getName());
-        return;
+        throw new MetricReporterException("Failed to create metric log directory for metrics " + this.getName(), ReporterType.METRIC, ReporterSinkType.FILE);
       }
 
       // Add a suffix to file name if specified in properties.
@@ -519,22 +554,19 @@ public class GobblinMetrics {
 
       LOGGER.info("Will start reporting metrics to directory " + metricsLogDir);
     } catch (IOException ioe) {
-      LOGGER.error("Failed to build file metric reporter for job " + this.id, ioe);
+      MetricReporterException e = new MetricReporterException("Failed to build file metric reporter for job " + this.id, ioe, ReporterType.METRIC, ReporterSinkType.FILE);
+      throw new MultiReporterException("Failed to create file metric reporter", Lists.newArrayList(e));
     }
   }
 
-  private void buildFileFailureEventReporter(Properties properties) {
-    if (!Boolean.valueOf(properties.getProperty(ConfigurationKeys.FAILURE_REPORTING_FILE_ENABLED_KEY,
-        ConfigurationKeys.DEFAULT_FAILURE_REPORTING_FILE_ENABLED))) {
+  private void buildFileFailureEventReporter(Properties properties)
+      throws MultiReporterException {
+    if ((!Boolean.valueOf(properties.getProperty(ConfigurationKeys.FAILURE_REPORTING_FILE_ENABLED_KEY,
+        ConfigurationKeys.DEFAULT_FAILURE_REPORTING_FILE_ENABLED)) || !properties.containsKey(ConfigurationKeys.FAILURE_LOG_DIR_KEY))) {
       return;
     }
-    LOGGER.info("Reporting failure to log files");
 
-    if (!properties.containsKey(ConfigurationKeys.FAILURE_LOG_DIR_KEY)) {
-      LOGGER.error(
-          "Not reporting failure to log files because " + ConfigurationKeys.FAILURE_LOG_DIR_KEY + " is undefined");
-      return;
-    }
+    LOGGER.info("Reporting failure to log files");
 
     try {
       String fsUri = properties.getProperty(ConfigurationKeys.FS_URI_KEY, ConfigurationKeys.LOCAL_FS_URI);
@@ -543,8 +575,7 @@ public class GobblinMetrics {
       // Each job gets its own log subdirectory
       Path failureLogDir = new Path(properties.getProperty(ConfigurationKeys.FAILURE_LOG_DIR_KEY), this.getName());
       if (!fs.exists(failureLogDir) && !fs.mkdirs(failureLogDir)) {
-        LOGGER.error("Failed to create failure log directory for metrics " + this.getName());
-        return;
+        throw new MetricReporterException("Failed to create failure log directory for metrics " + this.getName(), ReporterType.EVENT, ReporterSinkType.FILE);
       }
 
       // Add a suffix to file name if specified in properties.
@@ -562,7 +593,8 @@ public class GobblinMetrics {
 
       LOGGER.info("Will start reporting failure to directory " + failureLogDir);
     } catch (IOException ioe) {
-      LOGGER.error("Failed to build file failure event reporter for job " + this.id, ioe);
+      MetricReporterException e = new MetricReporterException("Failed to build file failure event reporter for job " + this.id, ioe, ReporterType.EVENT, ReporterSinkType.FILE);
+      throw new MultiReporterException("Failed to create failure file event reporter", Lists.newArrayList(e));
     }
   }
 
@@ -577,15 +609,33 @@ public class GobblinMetrics {
         convertRatesTo(TimeUnit.SECONDS).convertDurationsTo(TimeUnit.MILLISECONDS).build()));
   }
 
-  private void buildKafkaMetricReporter(Properties properties) {
+  private void buildKafkaMetricReporter(Properties properties)
+      throws MultiReporterException {
+    List<MetricReporterException> reporterExceptions = Lists.newArrayList();
     if (!Boolean.valueOf(properties.getProperty(ConfigurationKeys.METRICS_REPORTING_KAFKA_ENABLED_KEY,
         ConfigurationKeys.DEFAULT_METRICS_REPORTING_KAFKA_ENABLED))) {
       return;
     }
-    buildScheduledReporter(properties, ConfigurationKeys.DEFAULT_METRICS_REPORTING_KAFKA_REPORTER_CLASS, Optional.of("Kafka"));
+
+    try {
+      buildScheduledReporter(properties, ConfigurationKeys.DEFAULT_METRICS_REPORTING_KAFKA_REPORTER_CLASS);
+    } catch (MetricReporterException e) {
+      reporterExceptions.add(e);
+    }
+    try {
+      buildScheduledReporter(properties, ConfigurationKeys.DEFAULT_EVENTS_REPORTING_KAFKA_REPORTER_CLASS);
+    } catch (MetricReporterException e) {
+      reporterExceptions.add(e);
+    }
+    if (!reporterExceptions.isEmpty()) {
+      throw new MultiReporterException("Failed to start one or more Kafka reporters", reporterExceptions);
+    }
   }
 
-  private void buildGraphiteMetricReporter(Properties properties) {
+  private void buildGraphiteMetricReporter(Properties properties)
+      throws MultiReporterException {
+    List<MetricReporterException> reporterExceptionList = Lists.newArrayList();
+
     boolean metricsEnabled = PropertiesUtils
         .getPropAsBoolean(properties, ConfigurationKeys.METRICS_REPORTING_GRAPHITE_METRICS_ENABLED_KEY,
             ConfigurationKeys.DEFAULT_METRICS_REPORTING_GRAPHITE_METRICS_ENABLED);
@@ -608,8 +658,8 @@ public class GobblinMetrics {
       Preconditions.checkArgument(properties.containsKey(ConfigurationKeys.METRICS_REPORTING_GRAPHITE_HOSTNAME),
           "Graphite hostname is missing.");
     } catch (IllegalArgumentException exception) {
-      LOGGER.error("Not reporting to Graphite due to missing Graphite configuration(s).", exception);
-      return;
+      reporterExceptionList.add(new MetricReporterException("Missing Graphite configuration(s).", exception, ReporterType.METRIC_EVENT, ReporterSinkType.GRAPHITE));
+      throw new MultiReporterException("Failed to start one or more Graphite reporters", reporterExceptionList);
     }
 
     String hostname = properties.getProperty(ConfigurationKeys.METRICS_REPORTING_GRAPHITE_HOSTNAME);
@@ -637,7 +687,7 @@ public class GobblinMetrics {
             .withMetricsPrefix(prefix)
             .build(properties);
       } catch (IOException e) {
-        LOGGER.error("Failed to create Graphite metrics reporter. Will not report metrics to Graphite.", e);
+        reporterExceptionList.add(new MetricReporterException("Failed to create Graphite metrics reporter.", e, ReporterType.METRIC, ReporterSinkType.GRAPHITE));
       }
     }
 
@@ -659,12 +709,19 @@ public class GobblinMetrics {
         this.codahaleScheduledReporters.add(this.codahaleReportersCloser.register(eventReporter));
       }
       catch (IOException e) {
-        LOGGER.error("Failed to create Graphite event reporter. Will not report events to Graphite.", e);
+        reporterExceptionList.add(new MetricReporterException("Failed to create Graphite event reporter.", e, ReporterType.EVENT, ReporterSinkType.GRAPHITE));
       }
     }
+
+    if (!reporterExceptionList.isEmpty()) {
+      throw new MultiReporterException("Failed to create one or more Graphite Reporters", reporterExceptionList);
+    }
   }
 
-  private void buildInfluxDBMetricReporter(Properties properties) {
+  private void buildInfluxDBMetricReporter(Properties properties)
+      throws MultiReporterException {
+    List<MetricReporterException> reporterExceptionList = Lists.newArrayList();
+
     boolean metricsEnabled = PropertiesUtils
         .getPropAsBoolean(properties, ConfigurationKeys.METRICS_REPORTING_INFLUXDB_METRICS_ENABLED_KEY,
             ConfigurationKeys.DEFAULT_METRICS_REPORTING_INFLUXDB_METRICS_ENABLED);
@@ -687,8 +744,8 @@ public class GobblinMetrics {
       Preconditions.checkArgument(properties.containsKey(ConfigurationKeys.METRICS_REPORTING_INFLUXDB_DATABASE),
           "InfluxDB database name is missing.");
     } catch (IllegalArgumentException exception) {
-      LOGGER.error("Not reporting to InfluxDB due to missing InfluxDB configuration(s).", exception);
-      return;
+      reporterExceptionList.add(new MetricReporterException("Missing InfluxDB configuration(s)", exception, ReporterType.METRIC_EVENT, ReporterSinkType.INFLUXDB));
+      throw new MultiReporterException("Failed to start one or more InfluxDB reporters", reporterExceptionList);
     }
 
     String url = properties.getProperty(ConfigurationKeys.METRICS_REPORTING_INFLUXDB_URL);
@@ -715,7 +772,7 @@ public class GobblinMetrics {
             this.metricContext.getName()) // contains the current job id
             .build(properties);
       } catch (IOException e) {
-        LOGGER.error("Failed to create InfluxDB metrics reporter. Will not report metrics to InfluxDB.", e);
+        reporterExceptionList.add(new MetricReporterException("Failed to create InfluxDB metrics reporter.", e, ReporterType.METRIC, ReporterSinkType.INFLUXDB));
       }
     }
 
@@ -731,9 +788,13 @@ public class GobblinMetrics {
         this.codahaleScheduledReporters.add(this.codahaleReportersCloser.register(eventReporter));
       }
       catch (IOException e) {
-        LOGGER.error("Failed to create InfluxDB event reporter. Will not report events to InfluxDB.", e);
+        reporterExceptionList.add(new MetricReporterException("Failed to create InfluxDB event reporter.", e, ReporterType.EVENT, ReporterSinkType.INFLUXDB));
       }
     }
+
+    if (!reporterExceptionList.isEmpty()) {
+      throw new MultiReporterException("Failed to create one or more InfluxDB reporters", reporterExceptionList);
+    }
   }
 
   /**
@@ -741,7 +802,7 @@ public class GobblinMetrics {
    * {@link org.apache.gobblin.configuration.ConfigurationKeys#METRICS_CUSTOM_BUILDERS}. This allows users to specify custom
    * reporters for Gobblin runtime without having to modify the code.
    */
-  private void buildCustomMetricReporters(Properties properties) {
+  private void buildCustomMetricReporters(Properties properties, List<MetricReporterException> reporterExceptions) {
     String reporterClasses = properties.getProperty(ConfigurationKeys.METRICS_CUSTOM_BUILDERS);
 
     if (Strings.isNullOrEmpty(reporterClasses)) {
@@ -749,11 +810,16 @@ public class GobblinMetrics {
     }
 
     for (String reporterClass : Splitter.on(",").split(reporterClasses)) {
-      buildScheduledReporter(properties, reporterClass, Optional.<String>absent());
+      try {
+        buildScheduledReporter(properties, reporterClass);
+      } catch (MetricReporterException e) {
+        reporterExceptions.add(e);
+      }
     }
   }
 
-  private void buildScheduledReporter(Properties properties, String reporterClass, Optional<String> reporterSink) {
+  private void buildScheduledReporter(Properties properties, String reporterClass)
+      throws MetricReporterException {
     try {
       Class<?> clazz = Class.forName(reporterClass);
 
@@ -767,31 +833,28 @@ public class GobblinMetrics {
           return;
         }
         this.codahaleReportersCloser.register(scheduledReporter);
-        String reporterSinkMsg = reporterSink.isPresent()?"to " + reporterSink.get():"";
-        LOGGER.info("Will start reporting metrics " + reporterSinkMsg + " using " + reporterClass);
+        LOGGER.info("Will start reporting metrics using " + reporterClass);
         this.codahaleScheduledReporters.add(scheduledReporter);
-
       } else if (CustomReporterFactory.class.isAssignableFrom(clazz)) {
         CustomReporterFactory customReporterFactory = ((CustomReporterFactory) clazz.getConstructor().newInstance());
         customReporterFactory.newScheduledReporter(properties);
         LOGGER.info("Will start reporting metrics using " + reporterClass);
-
       } else {
-        throw new IllegalArgumentException("Class " + reporterClass +
+        throw new MetricReporterException("Class " + reporterClass +
             " specified by key " + ConfigurationKeys.METRICS_CUSTOM_BUILDERS + " must implement: "
-            + CustomCodahaleReporterFactory.class + " or " + CustomReporterFactory.class);
+            + CustomCodahaleReporterFactory.class + " or " + CustomReporterFactory.class, ReporterType.CUSTOM, ReporterSinkType.CUSTOM);
       }
     } catch (ClassNotFoundException exception) {
-      LOGGER.warn(String
+      throw new MetricReporterException(String
           .format("Failed to create metric reporter: requested CustomReporterFactory %s not found.", reporterClass),
-          exception);
-
+          exception, ReporterType.CUSTOM, ReporterSinkType.CUSTOM);
     } catch (NoSuchMethodException exception) {
-      LOGGER.warn(String.format("Failed to create metric reporter: requested CustomReporterFactory %s "
-          + "does not have parameterless constructor.", reporterClass), exception);
-
+      throw new MetricReporterException(String.format("Failed to create metric reporter: requested CustomReporterFactory %s "
+          + "does not have parameterless constructor.", reporterClass), exception, ReporterType.CUSTOM, ReporterSinkType.CUSTOM);
+    } catch (MetricReporterException exception) {
+      throw exception;
     } catch (Exception exception) {
-      LOGGER.warn("Could not create metric reporter from builder " + reporterClass + ".", exception);
+      throw new MetricReporterException("Could not create metric reporter from builder " + reporterClass + ".", exception, ReporterType.CUSTOM, ReporterSinkType.CUSTOM);
     }
   }
 }
