@@ -17,7 +17,27 @@
 
 package org.apache.gobblin.couchbase.writer;
 
+import com.couchbase.client.core.lang.Tuple;
+import com.couchbase.client.core.lang.Tuple2;
+import com.couchbase.client.core.message.ResponseStatus;
+import com.couchbase.client.core.message.kv.MutationToken;
+import com.couchbase.client.deps.io.netty.buffer.ByteBuf;
+import com.couchbase.client.java.Bucket;
+import com.couchbase.client.java.Cluster;
+import com.couchbase.client.java.CouchbaseCluster;
+import com.couchbase.client.java.auth.CertAuthenticator;
+import com.couchbase.client.java.document.AbstractDocument;
+import com.couchbase.client.java.document.RawJsonDocument;
+import com.couchbase.client.java.env.CouchbaseEnvironment;
+import com.couchbase.client.java.transcoder.Transcoder;
+import com.google.common.annotations.VisibleForTesting;
+import com.google.common.base.Preconditions;
+import com.google.gson.JsonElement;
+import com.google.gson.JsonObject;
+import com.google.gson.JsonParser;
+import com.typesafe.config.Config;
 import java.io.IOException;
+import java.nio.charset.StandardCharsets;
 import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.ArrayBlockingQueue;
@@ -27,31 +47,11 @@ import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicBoolean;
-
-import org.apache.commons.math3.util.Pair;
-
-import com.couchbase.client.core.lang.Tuple;
-import com.couchbase.client.core.lang.Tuple2;
-import com.couchbase.client.core.message.ResponseStatus;
-import com.couchbase.client.core.message.kv.MutationToken;
-import com.couchbase.client.deps.io.netty.buffer.ByteBuf;
-import com.couchbase.client.java.Bucket;
-import com.couchbase.client.java.Cluster;
-import com.couchbase.client.java.CouchbaseCluster;
-import com.couchbase.client.java.document.AbstractDocument;
-import com.couchbase.client.java.document.Document;
-import com.couchbase.client.java.document.RawJsonDocument;
-import com.couchbase.client.java.env.CouchbaseEnvironment;
-import com.couchbase.client.java.transcoder.Transcoder;
-import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Preconditions;
-import com.typesafe.config.Config;
-
+import java.util.stream.Collectors;
 import lombok.extern.slf4j.Slf4j;
-import rx.Observable;
-import rx.Subscriber;
-
+import org.apache.commons.math3.util.Pair;
 import org.apache.gobblin.couchbase.common.TupleDocument;
+import org.apache.gobblin.source.extractor.DataRecordException;
 import org.apache.gobblin.util.ConfigUtils;
 import org.apache.gobblin.writer.AsyncDataWriter;
 import org.apache.gobblin.writer.GenericWriteResponse;
@@ -61,6 +61,8 @@ import org.apache.gobblin.writer.WriteCallback;
 import org.apache.gobblin.writer.WriteResponse;
 import org.apache.gobblin.writer.WriteResponseFuture;
 import org.apache.gobblin.writer.WriteResponseMapper;
+import rx.Observable;
+import rx.Subscriber;
 
 
 /**
@@ -72,6 +74,11 @@ public class CouchbaseWriter<D extends AbstractDocument> implements AsyncDataWri
   private final Cluster _cluster;
   private final Bucket _bucket;
   private final long _operationTimeout;
+  private final int _documentTTL;
+  private final TimeUnit _documentTTLTimeUnits;
+  private final String _documentTTLOriginField;
+  private final TimeUnit _documentTTLOriginUnits;
+
   private final TimeUnit _operationTimeunit;
   private final WriteResponseMapper<D> _defaultWriteResponseMapper;
 
@@ -109,29 +116,41 @@ public class CouchbaseWriter<D extends AbstractDocument> implements AsyncDataWri
   public CouchbaseWriter(CouchbaseEnvironment couchbaseEnvironment, Config config) {
 
     List<String> hosts = ConfigUtils.getStringList(config, CouchbaseWriterConfigurationKeys.BOOTSTRAP_SERVERS);
+    boolean usesCertAuth = ConfigUtils.getBoolean(config, CouchbaseWriterConfigurationKeys.CERT_AUTH_ENABLED, false);
     String password = ConfigUtils.getString(config, CouchbaseWriterConfigurationKeys.PASSWORD, "");
 
+    log.info("Using hosts hosts: {}", hosts.stream().collect(Collectors.joining(",")));
+
+    _documentTTL = ConfigUtils.getInt(config, CouchbaseWriterConfigurationKeys.DOCUMENT_TTL, 0);
+    _documentTTLTimeUnits =
+        ConfigUtils.getTimeUnit(config, CouchbaseWriterConfigurationKeys.DOCUMENT_TTL_UNIT, CouchbaseWriterConfigurationKeys.DOCUMENT_TTL_UNIT_DEFAULT);
+    _documentTTLOriginField =
+        ConfigUtils.getString(config, CouchbaseWriterConfigurationKeys.DOCUMENT_TTL_ORIGIN_FIELD, null);
+    _documentTTLOriginUnits =
+        ConfigUtils.getTimeUnit(config, CouchbaseWriterConfigurationKeys.DOCUMENT_TTL_ORIGIN_FIELD_UNITS,
+            CouchbaseWriterConfigurationKeys.DOCUMENT_TTL_ORIGIN_FIELD_UNITS_DEFAULT);
+
     String bucketName = ConfigUtils.getString(config, CouchbaseWriterConfigurationKeys.BUCKET,
         CouchbaseWriterConfigurationKeys.BUCKET_DEFAULT);
 
     _cluster = CouchbaseCluster.create(couchbaseEnvironment, hosts);
 
-    if(!password.isEmpty()) {
-      _bucket = _cluster.openBucket(bucketName, password,
-          Collections.<Transcoder<? extends Document, ?>>singletonList(_tupleDocumentTranscoder));
+    if (usesCertAuth) {
+      _cluster.authenticate(CertAuthenticator.INSTANCE);
+      _bucket = _cluster.openBucket(bucketName, Collections.singletonList(_tupleDocumentTranscoder));
+    } else if (password.isEmpty()) {
+      _bucket = _cluster.openBucket(bucketName, Collections.singletonList(_tupleDocumentTranscoder));
     } else {
-      _bucket = _cluster.openBucket(bucketName,
-          Collections.<Transcoder<? extends Document, ?>>singletonList(_tupleDocumentTranscoder));
+      _bucket = _cluster.openBucket(bucketName, password, Collections.singletonList(_tupleDocumentTranscoder));
     }
-
     _operationTimeout = ConfigUtils.getLong(config, CouchbaseWriterConfigurationKeys.OPERATION_TIMEOUT_MILLIS,
         CouchbaseWriterConfigurationKeys.OPERATION_TIMEOUT_DEFAULT);
     _operationTimeunit = TimeUnit.MILLISECONDS;
 
     _defaultWriteResponseMapper = new GenericWriteResponseWrapper<>();
 
-    log.info("Couchbase writer configured with: hosts: {}, bucketName: {}, operationTimeoutInMillis: {}",
-        hosts, bucketName, _operationTimeout);
+    log.info("Couchbase writer configured with: hosts: {}, bucketName: {}, operationTimeoutInMillis: {}", hosts,
+        bucketName, _operationTimeout);
   }
 
   @VisibleForTesting
@@ -152,7 +171,13 @@ public class CouchbaseWriter<D extends AbstractDocument> implements AsyncDataWri
     if (record instanceof TupleDocument) {
       ((TupleDocument) record).content().value1().retain();
     }
-    Observable<D> observable = _bucket.async().upsert(record);
+    Observable<D> observable;
+    try {
+      observable = _bucket.async().upsert(setDocumentTTL(record));
+    } catch (DataRecordException e) {
+      throw new RuntimeException("Caught exception trying to set TTL of the document", e);
+    }
+
     if (callback == null) {
       return new WriteResponseFuture<>(
           observable.timeout(_operationTimeout, _operationTimeunit).toBlocking().toFuture(),
@@ -179,10 +204,9 @@ public class CouchbaseWriter<D extends AbstractDocument> implements AsyncDataWri
         }
 
         @Override
-        public WriteResponse get()
-            throws InterruptedException, ExecutionException {
+        public WriteResponse get() throws InterruptedException, ExecutionException {
           Pair<WriteResponse, Throwable> writeResponseThrowablePair = writeResponseQueue.take();
-          return getWriteResponseorThrow(writeResponseThrowablePair);
+          return getWriteResponseOrThrow(writeResponseThrowablePair);
         }
 
         @Override
@@ -192,49 +216,47 @@ public class CouchbaseWriter<D extends AbstractDocument> implements AsyncDataWri
           if (writeResponseThrowablePair == null) {
             throw new TimeoutException("Timeout exceeded while waiting for future to be done");
           } else {
-            return getWriteResponseorThrow(writeResponseThrowablePair);
+            return getWriteResponseOrThrow(writeResponseThrowablePair);
           }
         }
       };
 
-      observable.timeout(_operationTimeout, _operationTimeunit)
-          .subscribe(new Subscriber<D>() {
-            @Override
-            public void onCompleted() {
-            }
+      observable.timeout(_operationTimeout, _operationTimeunit).subscribe(new Subscriber<D>() {
+        @Override
+        public void onCompleted() {
+        }
 
-            @Override
-            public void onError(Throwable e) {
-              callbackFired.set(true);
-              writeResponseQueue.add(new Pair<WriteResponse, Throwable>(null, e));
-              callback.onFailure(e);
-            }
+        @Override
+        public void onError(Throwable e) {
+          callbackFired.set(true);
+          writeResponseQueue.add(new Pair<WriteResponse, Throwable>(null, e));
+          callback.onFailure(e);
+        }
 
-            @Override
-            public void onNext(D doc) {
-              try {
-                callbackFired.set(true);
-                WriteResponse writeResponse = new GenericWriteResponse<D>(doc);
-                writeResponseQueue.add(new Pair<WriteResponse, Throwable>(writeResponse, null));
-                callback.onSuccess(writeResponse);
-              } finally {
-                if (doc instanceof TupleDocument) {
-                  ((TupleDocument) doc).content().value1().release();
-                }
-              }
+        @Override
+        public void onNext(D doc) {
+          try {
+            callbackFired.set(true);
+            WriteResponse writeResponse = new GenericWriteResponse<D>(doc);
+            writeResponseQueue.add(new Pair<WriteResponse, Throwable>(writeResponse, null));
+            callback.onSuccess(writeResponse);
+          } finally {
+            if (doc instanceof TupleDocument) {
+              ((TupleDocument) doc).content().value1().release();
             }
-          });
+          }
+        }
+      });
       return writeResponseFuture;
     }
   }
 
   @Override
-  public void flush()
-      throws IOException {
+  public void flush() throws IOException {
 
   }
 
-  private WriteResponse getWriteResponseorThrow(Pair<WriteResponse, Throwable> writeResponseThrowablePair)
+  private WriteResponse getWriteResponseOrThrow(Pair<WriteResponse, Throwable> writeResponseThrowablePair)
       throws ExecutionException {
     if (writeResponseThrowablePair.getFirst() != null) {
       return writeResponseThrowablePair.getFirst();
@@ -246,17 +268,71 @@ public class CouchbaseWriter<D extends AbstractDocument> implements AsyncDataWri
   }
 
   @Override
-  public void cleanup()
-      throws IOException {
+  public void cleanup() throws IOException {
 
   }
 
-  @Override
-  public WriteResponse write(D record)
-      throws IOException {
+  /**
+   * Returns a new document with 32 bit (int) timestamp expiration date for the document. Note this is a current limitation in couchbase.
+   * This approach should work for documents that do not expire until 2038. This should be enough headroom for couchbase
+   * to reimplement the design.
+   * Source: https://forums.couchbase.com/t/document-expiry-in-seconds-or-a-timestamp/6519/6
+   * @param record
+   * @return
+   */
+  private D setDocumentTTL(D record) throws DataRecordException {
+    boolean recordIsTupleDocument = record instanceof TupleDocument;
+    boolean recordIsJsonDocument = record instanceof RawJsonDocument;
 
+    long ttlSpanSec = TimeUnit.SECONDS.convert(_documentTTL, _documentTTLTimeUnits);
+    long eventOriginSec = 0;
+    String dataJson = null;
+    if (_documentTTL == 0) {
+      return record;
+    } else if (_documentTTLOriginField != null && !_documentTTLOriginField.isEmpty()) {
+
+      if (recordIsTupleDocument) {
+        ByteBuf dataByteBuffer = ((Tuple2<ByteBuf, Integer>) record.content()).value1();
+        dataJson = new String(dataByteBuffer.array(), StandardCharsets.UTF_8);
+      } else {
+        dataJson = (String) record.content();
+      }
+      JsonElement jsonDataRootElement = new JsonParser().parse(dataJson);
+      if (!jsonDataRootElement.isJsonObject()) {
+        throw new DataRecordException(
+            String.format("Document TTL Field is set but the record's value is not a valid json object.: '%s'",
+                jsonDataRootElement.toString()));
+      }
+      JsonObject jsonDataRoot = jsonDataRootElement.getAsJsonObject();
+      long documentTTLOrigin = jsonDataRoot.get(_documentTTLOriginField).getAsLong();
+      eventOriginSec = TimeUnit.SECONDS.convert(documentTTLOrigin, _documentTTLOriginUnits);
+    } else {
+      eventOriginSec = System.currentTimeMillis() / 1000;
+    }
     try {
-      D doc = _bucket.upsert(record);
+      int expiration = Math.toIntExact(ttlSpanSec + eventOriginSec);
+      if (recordIsTupleDocument) {
+        return (D) _tupleDocumentTranscoder.newDocument(record.id(), expiration,
+            (Tuple2<ByteBuf, Integer>) record.content(), record.cas(), record.mutationToken());
+      } else if (recordIsJsonDocument) {
+        return (D) RawJsonDocument.create(record.id(), expiration, (String) record.content(), record.cas(),
+            record.mutationToken());
+      } else {
+        throw new RuntimeException(" Only TupleDocument and RawJsonDocument documents are supported");
+      }
+    } catch (ArithmeticException e) {
+      throw new RuntimeException(
+          "There was an overflow calculating the expiry timestamp. couchbase currently only supports expiry until January 19, 2038 03:14:07 GMT",
+          e);
+    }
+  }
+
+  @Override
+  public WriteResponse write(D record) throws IOException {
+    try {
+      D doc = _bucket.upsert(setDocumentTTL(record));
+
+      Preconditions.checkNotNull(doc);
       return new GenericWriteResponse(doc);
     } catch (Exception e) {
       throw new IOException("Failed to write to Couchbase cluster", e);
