@@ -26,6 +26,7 @@ import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.sql.Timestamp;
 import java.sql.Types;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
@@ -306,7 +307,7 @@ public abstract class JdbcExtractor extends QueryBasedExtractor<JsonArray, JsonE
         ConfigurationKeys.ENABLE_DELIMITED_IDENTIFIER, ConfigurationKeys.DEFAULT_ENABLE_DELIMITED_IDENTIFIER);
     JsonObject defaultWatermark = this.getDefaultWatermark();
     String derivedWatermarkColumnName = defaultWatermark.get("columnName").getAsString();
-    this.setSampleRecordCount(this.exractSampleRecordCountFromQuery(inputQuery));
+    this.setSampleRecordCount(this.extractSampleRecordCountFromQuery(inputQuery));
     inputQuery = this.removeSampleClauseFromQuery(inputQuery);
     JsonArray targetSchema = new JsonArray();
     List<String> headerColumns = new ArrayList<>();
@@ -959,6 +960,9 @@ public abstract class JdbcExtractor extends QueryBasedExtractor<JsonArray, JsonE
       int batchSize = this.workUnitState.getPropAsInt(ConfigurationKeys.SOURCE_QUERYBASED_FETCH_SIZE, 0);
       batchSize = (batchSize == 0 ? ConfigurationKeys.DEFAULT_SOURCE_FETCH_SIZE : batchSize);
 
+      String sourceConnProps = this.workUnitState.getProp(ConfigurationKeys.SOURCE_CONN_PROPERTIES);
+      boolean convertZeroDateTime = sourceConnProps != null && sourceConnProps.contains("zeroDateTimeBehavior");
+
       int recordCount = 0;
       while (resultset.next()) {
 
@@ -967,8 +971,7 @@ public abstract class JdbcExtractor extends QueryBasedExtractor<JsonArray, JsonE
 
         for (int i = 1; i < numColumns + 1; i++) {
           final String columnName = this.getHeaderRecord().get(i - 1);
-          jsonObject.addProperty(columnName, parseColumnAsString(resultset, resultsetMetadata, i));
-
+          jsonObject.addProperty(columnName, parseColumnAsString(resultset, resultsetMetadata, i, convertZeroDateTime));
         }
 
         recordSet.add(jsonObject);
@@ -1028,7 +1031,7 @@ public abstract class JdbcExtractor extends QueryBasedExtractor<JsonArray, JsonE
    * treat tinyint(1) as BIT.
    *
    * Currently, {@link MysqlExtractor#getDataTypeMap()} uses the information_schema to check types.
-   * That does not do the above conversion. {@link #parseColumnAsString(ResultSet, ResultSetMetaData, int)}
+   * That does not do the above conversion. {@link #parseColumnAsString(ResultSet, ResultSetMetaData, int, boolean)}
    * which does the above type mapping.
    *
    * On the other hand, SqlServerExtractor treats BIT columns as Booleans. So we can be in a bind
@@ -1041,7 +1044,8 @@ public abstract class JdbcExtractor extends QueryBasedExtractor<JsonArray, JsonE
     return true;
   }
 
-  private String parseColumnAsString(final ResultSet resultset, final ResultSetMetaData resultsetMetadata, int i)
+  private String parseColumnAsString(final ResultSet resultset, final ResultSetMetaData resultsetMetadata, int i,
+      boolean convertZeroDateTime)
       throws SQLException {
 
     if (isBlob(resultsetMetadata.getColumnType(i))) {
@@ -1055,6 +1059,18 @@ public abstract class JdbcExtractor extends QueryBasedExtractor<JsonArray, JsonE
         && convertBitToBoolean()) {
       return Boolean.toString(resultset.getBoolean(i));
     }
+
+    // Workaround for when `zeroDateTimeBehavior` is set
+    // returns null or a rounded timestamp instead of "0000-00-00 00:00:00" for zero timestamps
+    if (convertZeroDateTime && isTimestamp(resultsetMetadata.getColumnType(i))) {
+        Timestamp ts = resultset.getTimestamp(i);
+        if (ts == null) {
+          return null;
+        } else {
+          return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(ts);
+        }
+    }
+
     return resultset.getString(i);
   }
 
@@ -1066,6 +1082,10 @@ public abstract class JdbcExtractor extends QueryBasedExtractor<JsonArray, JsonE
     return columnType == Types.CLOB;
   }
 
+  private static boolean isTimestamp(int columnType) {
+    return columnType == Types.TIMESTAMP || columnType == Types.TIMESTAMP_WITH_TIMEZONE;
+  }
+
   protected static Command getCommand(String query, JdbcCommandType commandType) {
     return new JdbcCommand().build(Arrays.asList(query), commandType);
   }
