@@ -231,7 +231,7 @@ public class MysqlExtractor extends JdbcExtractor {
   }
 
   @Override
-  public long exractSampleRecordCountFromQuery(String query) {
+  public long extractSampleRecordCountFromQuery(String query) {
     if (StringUtils.isBlank(query)) {
       return SAMPLERECORDCOUNT;
     }
