@@ -203,7 +203,7 @@ public class SqlServerExtractor extends JdbcExtractor {
   }
 
   @Override
-  public long exractSampleRecordCountFromQuery(String query) {
+  public long extractSampleRecordCountFromQuery(String query) {
     if (StringUtils.isBlank(query)) {
       return SAMPLERECORDCOUNT;
     }
