@@ -20,6 +20,7 @@ package org.apache.gobblin.service;
 import java.io.Closeable;
 import java.io.IOException;
 import java.util.Collections;
+import java.util.Map;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -59,13 +60,17 @@ public class FlowConfigClient implements Closeable {
    * @param serverUri address and port of the REST server
    */
   public FlowConfigClient(String serverUri) {
+    this(serverUri, Collections.emptyMap());
+  }
+
+  public FlowConfigClient(String serverUri, Map<String, String> properties) {
     LOG.debug("FlowConfigClient with serverUri " + serverUri);
 
     _httpClientFactory = Optional.of(new HttpClientFactory());
-    Client r2Client = new TransportClientAdapter(_httpClientFactory.get().getClient(Collections.<String, String>emptyMap()));
+    Client r2Client = new TransportClientAdapter(_httpClientFactory.get().getClient(properties));
     _restClient = Optional.of(new RestClient(r2Client, serverUri));
 
-    _flowconfigsRequestBuilders = new FlowconfigsRequestBuilders();
+    _flowconfigsRequestBuilders = createRequestBuilders();
   }
 
   /**
@@ -78,7 +83,13 @@ public class FlowConfigClient implements Closeable {
     _httpClientFactory = Optional.absent();
     _restClient = Optional.of(restClient);
 
-    _flowconfigsRequestBuilders = new FlowconfigsRequestBuilders();
+    _flowconfigsRequestBuilders = createRequestBuilders();
+  }
+
+  // Clients using different service name can override this method
+  // RequestBuilders decide the name of the service requests go to.
+  protected FlowconfigsRequestBuilders createRequestBuilders() {
+    return new FlowconfigsRequestBuilders();
   }
 
   /**
@@ -129,8 +140,7 @@ public class FlowConfigClient implements Closeable {
    */
   public FlowConfig getFlowConfig(FlowId flowId)
       throws RemoteInvocationException {
-    LOG.debug("getFlowConfig with groupName " + flowId.getFlowGroup() + " flowName " +
-        flowId.getFlowName());
+    LOG.debug("getFlowConfig with groupName " + flowId.getFlowGroup() + " flowName " + flowId.getFlowName());
 
     GetRequest<FlowConfig> getRequest = _flowconfigsRequestBuilders.get()
         .id(new ComplexResourceKey<>(flowId, new EmptyRecord())).build();
