@@ -34,15 +34,18 @@ import com.google.inject.Binder;
 import com.google.inject.Guice;
 import com.google.inject.Injector;
 import com.google.inject.Module;
-import com.google.inject.name.Names;
 import com.linkedin.restli.server.resources.BaseResource;
 
 import org.apache.gobblin.configuration.State;
 import org.apache.gobblin.metastore.StateStore;
 import org.apache.gobblin.restli.EmbeddedRestliServer;
+import org.apache.gobblin.runtime.troubleshooter.MultiContextIssueRepository;
 import org.apache.gobblin.service.monitoring.FlowStatusGenerator;
 import org.apache.gobblin.service.monitoring.JobStatusRetriever;
 
+import static org.mockito.Mockito.mock;
+
+
 @Test(groups = { "gobblin.service" }, singleThreaded = true)
 public class FlowStatusTest {
   private FlowStatusClient _client;
@@ -50,6 +53,11 @@ public class FlowStatusTest {
   private List<List<org.apache.gobblin.service.monitoring.JobStatus>> _listOfJobStatusLists;
 
   class TestJobStatusRetriever extends JobStatusRetriever {
+
+    protected TestJobStatusRetriever(MultiContextIssueRepository issueRepository) {
+      super(issueRepository);
+    }
+
     @Override
     public Iterator<org.apache.gobblin.service.monitoring.JobStatus> getJobStatusesForFlowExecution(String flowName,
         String flowGroup, long flowExecutionId) {
@@ -82,14 +90,13 @@ public class FlowStatusTest {
 
   @BeforeClass
   public void setUp() throws Exception {
-    JobStatusRetriever jobStatusRetriever = new TestJobStatusRetriever();
-    final FlowStatusGenerator flowStatusGenerator =
-        FlowStatusGenerator.builder().jobStatusRetriever(jobStatusRetriever).build();
+    JobStatusRetriever jobStatusRetriever = new TestJobStatusRetriever(mock(MultiContextIssueRepository.class));
+    final FlowStatusGenerator flowStatusGenerator = new FlowStatusGenerator(jobStatusRetriever);
 
     Injector injector = Guice.createInjector(new Module() {
        @Override
        public void configure(Binder binder) {
-         binder.bind(FlowStatusGenerator.class).annotatedWith(Names.named(FlowStatusResource.FLOW_STATUS_GENERATOR_INJECT_NAME))
+         binder.bind(FlowStatusGenerator.class)
              .toInstance(flowStatusGenerator);
        }
     });
@@ -110,24 +117,33 @@ public class FlowStatusTest {
    */
   @Test
   public void testFindLatest() throws Exception {
-    org.apache.gobblin.service.monitoring.JobStatus js1 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup("jgroup1").jobName("job1").startTime(1000L).endTime(5000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 1")
-        .processedCount(100).jobExecutionId(1).lowWatermark("watermark:1").highWatermark("watermark:2").build();
-    org.apache.gobblin.service.monitoring.JobStatus fs1 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY).endTime(5000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).build();
-    org.apache.gobblin.service.monitoring.JobStatus js2 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup("jgroup1").jobName("job1").jobTag("dataset1").startTime(2000L).endTime(6000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(1).message("Test message 2")
-        .processedCount(200).jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").build();
-    org.apache.gobblin.service.monitoring.JobStatus js3 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup("jgroup1").jobName("job2").jobTag("dataset2").startTime(2000L).endTime(6000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(1).message("Test message 3")
-        .processedCount(200).jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").build();
-    org.apache.gobblin.service.monitoring.JobStatus fs2 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY).endTime(7000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(1).message("Flow message").build();
+    org.apache.gobblin.service.monitoring.JobStatus js1 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup("jgroup1").jobName("job1").startTime(1000L).endTime(5000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 1").processedCount(100)
+            .jobExecutionId(1).lowWatermark("watermark:1").highWatermark("watermark:2").issues(Collections.emptyList())
+            .build();
+    org.apache.gobblin.service.monitoring.JobStatus fs1 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY).endTime(5000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).issues(Collections.emptyList()).build();
+    org.apache.gobblin.service.monitoring.JobStatus js2 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup("jgroup1").jobName("job1").jobTag("dataset1").startTime(2000L).endTime(6000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(1).message("Test message 2").processedCount(200)
+            .jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").issues(Collections.emptyList())
+            .build();
+    org.apache.gobblin.service.monitoring.JobStatus js3 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup("jgroup1").jobName("job2").jobTag("dataset2").startTime(2000L).endTime(6000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(1).message("Test message 3").processedCount(200)
+            .jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").issues(Collections.emptyList())
+            .build();
+    org.apache.gobblin.service.monitoring.JobStatus fs2 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY).endTime(7000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(1).message("Flow message")
+            .issues(Collections.emptyList()).build();
     List<org.apache.gobblin.service.monitoring.JobStatus> jobStatusList1 = Lists.newArrayList(js1, fs1);
     List<org.apache.gobblin.service.monitoring.JobStatus> jobStatusList2 = Lists.newArrayList(js2, js3, fs2);
     _listOfJobStatusLists = Lists.newArrayList();
@@ -172,17 +188,23 @@ public class FlowStatusTest {
    */
   @Test
   public void testGetCompleted() throws Exception {
-    org.apache.gobblin.service.monitoring.JobStatus js1 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup("jgroup1").jobName("job1").startTime(1000L).endTime(5000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 1")
-        .processedCount(100).jobExecutionId(1).lowWatermark("watermark:1").highWatermark("watermark:2").build();
-    org.apache.gobblin.service.monitoring.JobStatus js2 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup("jgroup1").jobName("job2").startTime(2000L).endTime(6000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 2")
-        .processedCount(200).jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").build();
-    org.apache.gobblin.service.monitoring.JobStatus fs1 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY).endTime(7000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Flow message").build();
+    org.apache.gobblin.service.monitoring.JobStatus js1 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup("jgroup1").jobName("job1").startTime(1000L).endTime(5000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 1").processedCount(100)
+            .jobExecutionId(1).lowWatermark("watermark:1").highWatermark("watermark:2").issues(Collections.emptyList())
+            .build();
+    org.apache.gobblin.service.monitoring.JobStatus js2 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup("jgroup1").jobName("job2").startTime(2000L).endTime(6000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 2").processedCount(200)
+            .jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").issues(Collections.emptyList())
+            .build();
+    org.apache.gobblin.service.monitoring.JobStatus fs1 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY).endTime(7000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Flow message")
+            .issues(Collections.emptyList()).build();
     List<org.apache.gobblin.service.monitoring.JobStatus> jobStatusList = Lists.newArrayList(js1, js2, fs1);
     _listOfJobStatusLists = Lists.newArrayList();
     _listOfJobStatusLists.add(jobStatusList);
@@ -215,17 +237,23 @@ public class FlowStatusTest {
    */
   @Test
   public void testGetRunning() throws Exception {
-    org.apache.gobblin.service.monitoring.JobStatus js1 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup("jgroup1").jobName("job1").startTime(1000L).endTime(5000L)
-        .eventName(ExecutionStatus.RUNNING.name()).flowExecutionId(0).message("Test message 1").processedCount(100)
-        .jobExecutionId(1).lowWatermark("watermark:1").highWatermark("watermark:2").build();
-    org.apache.gobblin.service.monitoring.JobStatus js2 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup("jgroup1").jobName("job2").startTime(2000L).endTime(6000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 2")
-        .processedCount(200).jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").build();
-    org.apache.gobblin.service.monitoring.JobStatus fs1 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY)
-        .eventName(ExecutionStatus.RUNNING.name()).flowExecutionId(0).message("Flow message").build();
+    org.apache.gobblin.service.monitoring.JobStatus js1 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup("jgroup1").jobName("job1").startTime(1000L).endTime(5000L)
+            .eventName(ExecutionStatus.RUNNING.name()).flowExecutionId(0).message("Test message 1").processedCount(100)
+            .jobExecutionId(1).lowWatermark("watermark:1").highWatermark("watermark:2").issues(Collections.emptyList())
+            .build();
+    org.apache.gobblin.service.monitoring.JobStatus js2 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup("jgroup1").jobName("job2").startTime(2000L).endTime(6000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 2").processedCount(200)
+            .jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").issues(Collections.emptyList())
+            .build();
+    org.apache.gobblin.service.monitoring.JobStatus fs1 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY)
+            .eventName(ExecutionStatus.RUNNING.name()).flowExecutionId(0).message("Flow message")
+            .issues(Collections.emptyList()).build();
     List<org.apache.gobblin.service.monitoring.JobStatus> jobStatusList = Lists.newArrayList(js1, js2, fs1);
     _listOfJobStatusLists = Lists.newArrayList();
     _listOfJobStatusLists.add(jobStatusList);
@@ -258,17 +286,23 @@ public class FlowStatusTest {
    */
   @Test
   public void testGetFailed() throws Exception {
-    org.apache.gobblin.service.monitoring.JobStatus js1 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup("jgroup1").jobName("job1").startTime(1000L).endTime(5000L)
-        .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 1")
-        .processedCount(100).jobExecutionId(1).lowWatermark("watermark:1").highWatermark("watermark:2").build();
-    org.apache.gobblin.service.monitoring.JobStatus js2 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup("jgroup1").jobName("job2").startTime(2000L).endTime(6000L)
-        .eventName(ExecutionStatus.FAILED.name()).flowExecutionId(0).message("Test message 2")
-        .processedCount(200).jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").build();
-    org.apache.gobblin.service.monitoring.JobStatus fs1 = org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1")
-        .flowName("flow1").jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY).endTime(7000L)
-        .eventName(ExecutionStatus.FAILED.name()).flowExecutionId(0).message("Flow message").build();
+    org.apache.gobblin.service.monitoring.JobStatus js1 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup("jgroup1").jobName("job1").startTime(1000L).endTime(5000L)
+            .eventName(ExecutionStatus.COMPLETE.name()).flowExecutionId(0).message("Test message 1").processedCount(100)
+            .jobExecutionId(1).lowWatermark("watermark:1").highWatermark("watermark:2").issues(Collections.emptyList())
+            .build();
+    org.apache.gobblin.service.monitoring.JobStatus js2 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup("jgroup1").jobName("job2").startTime(2000L).endTime(6000L)
+            .eventName(ExecutionStatus.FAILED.name()).flowExecutionId(0).message("Test message 2").processedCount(200)
+            .jobExecutionId(2).lowWatermark("watermark:2").highWatermark("watermark:3").issues(Collections.emptyList())
+            .build();
+    org.apache.gobblin.service.monitoring.JobStatus fs1 =
+        org.apache.gobblin.service.monitoring.JobStatus.builder().flowGroup("fgroup1").flowName("flow1")
+            .jobGroup(JobStatusRetriever.NA_KEY).jobName(JobStatusRetriever.NA_KEY).endTime(7000L)
+            .eventName(ExecutionStatus.FAILED.name()).flowExecutionId(0).message("Flow message")
+            .issues(Collections.emptyList()).build();
     List<org.apache.gobblin.service.monitoring.JobStatus> jobStatusList = Lists.newArrayList(js1, js2, fs1);
     _listOfJobStatusLists = Lists.newArrayList();
     _listOfJobStatusLists.add(jobStatusList);
