@@ -17,10 +17,13 @@
 
 package org.apache.gobblin.runtime;
 
+import com.google.common.annotations.VisibleForTesting;
 import java.io.IOException;
+import java.net.Authenticator;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
@@ -30,6 +33,8 @@ import java.util.concurrent.Executors;
 import java.util.concurrent.TimeUnit;
 
 import org.apache.commons.lang.StringUtils;
+
+import org.apache.gobblin.service.ServiceConfigKeys;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.slf4j.MDC;
@@ -40,6 +45,7 @@ import com.google.common.base.Function;
 import com.google.common.base.Optional;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Predicate;
+import com.google.common.base.Strings;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
@@ -60,6 +66,7 @@ import org.apache.gobblin.commit.DeliverySemantics;
 import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.gobblin.configuration.WorkUnitState;
 import org.apache.gobblin.converter.initializer.ConverterInitializerFactory;
+import org.apache.gobblin.destination.DestinationDatasetHandlerService;
 import org.apache.gobblin.metrics.ContextAwareGauge;
 import org.apache.gobblin.metrics.GobblinMetrics;
 import org.apache.gobblin.metrics.GobblinMetricsRegistry;
@@ -83,6 +90,9 @@ import org.apache.gobblin.runtime.locks.JobLock;
 import org.apache.gobblin.runtime.locks.JobLockEventListener;
 import org.apache.gobblin.runtime.locks.JobLockException;
 import org.apache.gobblin.runtime.locks.LegacyJobLockFactoryManager;
+import org.apache.gobblin.runtime.troubleshooter.AutomaticTroubleshooter;
+import org.apache.gobblin.runtime.troubleshooter.AutomaticTroubleshooterFactory;
+import org.apache.gobblin.runtime.troubleshooter.IssueRepository;
 import org.apache.gobblin.runtime.util.JobMetrics;
 import org.apache.gobblin.source.Source;
 import org.apache.gobblin.source.WorkUnitStreamSource;
@@ -98,6 +108,7 @@ import org.apache.gobblin.util.Id;
 import org.apache.gobblin.util.JobLauncherUtils;
 import org.apache.gobblin.util.ParallelRunner;
 import org.apache.gobblin.util.PropertiesUtils;
+import org.apache.gobblin.util.reflection.GobblinConstructorUtils;
 import org.apache.gobblin.writer.initializer.WriterInitializerFactory;
 
 
@@ -168,6 +179,8 @@ public abstract class AbstractJobLauncher implements JobLauncher {
   // Used to generate additional metadata to emit in timing events
   private final MultiEventMetadataGenerator multiEventMetadataGenerator;
 
+  private final AutomaticTroubleshooter troubleshooter;
+
   public AbstractJobLauncher(Properties jobProps, List<? extends Tag<?>> metadataTags)
       throws Exception {
     this(jobProps, metadataTags, null);
@@ -184,6 +197,9 @@ public abstract class AbstractJobLauncher implements JobLauncher {
     clusterNameTags.addAll(Tag.fromMap(ClusterNameTags.getClusterNameTags()));
     GobblinMetrics.addCustomTagsToProperties(jobProps, clusterNameTags);
 
+    troubleshooter = AutomaticTroubleshooterFactory.createForJob(ConfigUtils.propertiesToConfig(jobProps));
+    troubleshooter.start();
+
     // Make a copy for both the system and job configuration properties and resolve the job-template if any.
     this.jobProps = new Properties();
     this.jobProps.putAll(jobProps);
@@ -195,11 +211,13 @@ public abstract class AbstractJobLauncher implements JobLauncher {
     }
 
     try {
+      setDefaultAuthenticator(this.jobProps);
+
       if (instanceBroker == null) {
         instanceBroker = createDefaultInstanceBroker(jobProps);
       }
 
-      this.jobContext = new JobContext(this.jobProps, LOG, instanceBroker);
+      this.jobContext = new JobContext(this.jobProps, LOG, instanceBroker, troubleshooter.getIssueRepository());
       this.eventBus.register(this.jobContext);
 
       this.cancellationExecutor = Executors.newSingleThreadExecutor(
@@ -230,6 +248,17 @@ public abstract class AbstractJobLauncher implements JobLauncher {
     }
   }
 
+  /**
+   * Set default {@link Authenticator} to the one provided in {@link ConfigurationKeys#DEFAULT_AUTHENTICATOR_CLASS},
+   * calling the constructor using the provided {@link Properties}
+   */
+  public static void setDefaultAuthenticator(Properties properties) {
+    String authenticatorClass = properties.getProperty(ConfigurationKeys.DEFAULT_AUTHENTICATOR_CLASS);
+    if (!Strings.isNullOrEmpty(authenticatorClass)) {
+      Authenticator authenticator = GobblinConstructorUtils.invokeConstructor(Authenticator.class, authenticatorClass, properties);
+      Authenticator.setDefault(authenticator);
+    }
+  }
 
   /**
    * To supporting 'gobblin.template.uri' in any types of jobLauncher, place this resolution as a public-static method
@@ -438,6 +467,22 @@ public abstract class AbstractJobLauncher implements JobLauncher {
           return;
         }
 
+        // calculation of total bytes to copy in a job used to track a job's copy progress
+        if (jobState.getPropAsBoolean(ConfigurationKeys.REPORT_JOB_PROGRESS, ConfigurationKeys.DEFAULT_REPORT_JOB_PROGRESS)) {
+          if (workUnitStream.isSafeToMaterialize()) {
+            long totalSizeInBytes = sumWorkUnitsSizes(workUnitStream);
+            this.jobContext.getJobState().setProp(ServiceConfigKeys.TOTAL_WORK_UNIT_SIZE, totalSizeInBytes);
+          } else {
+            LOG.warn("Property " + ConfigurationKeys.REPORT_JOB_PROGRESS + " is turned on, but "
+                + "progress cannot be reported for infinite work unit streams. Turn off property "
+                + ConfigurationKeys.REPORT_JOB_PROGRESS);
+          }
+        }
+
+        // Perform work needed before writing is done
+        Boolean canCleanUp = this.canCleanStagingData(this.jobContext.getJobState());
+        closer.register(new DestinationDatasetHandlerService(jobState, canCleanUp, this.eventSubmitter))
+            .executeHandlers(workUnitStream);
         //Initialize writer and converter(s)
         closer.register(WriterInitializerFactory.newInstace(jobState, workUnitStream)).initialize();
         closer.register(ConverterInitializerFactory.newInstance(jobState, workUnitStream)).initialize();
@@ -533,6 +578,14 @@ public abstract class AbstractJobLauncher implements JobLauncher {
         LOG.error(errMsg + ": " + t, t);
         this.jobContext.getJobState().setJobFailureException(t);
       } finally {
+        try {
+          troubleshooter.refineIssues();
+          troubleshooter.logIssueSummary();
+          troubleshooter.reportJobIssuesAsEvents(eventSubmitter);
+        } catch (Exception e) {
+          LOG.error("Failed to report issues", e);
+        }
+
         try {
           TimingEvent jobCleanupTimer = this.eventSubmitter.getTimingEvent(TimingEvent.LauncherTimings.JOB_CLEANUP);
           cleanupStagingData(jobState);
@@ -558,45 +611,45 @@ public abstract class AbstractJobLauncher implements JobLauncher {
                 jobListener.onJobFailure(jobContext);
               }
             });
+          } else {
+            for (JobState.DatasetState datasetState : this.jobContext.getDatasetStatesByUrns().values()) {
+              // Set the overall job state to FAILED if the job failed to process any dataset
+              if (datasetState.getState() == JobState.RunningState.FAILED) {
+                jobState.setState(JobState.RunningState.FAILED);
+                LOG.warn("At least one dataset state is FAILED. Setting job state to FAILED.");
+                break;
+              }
+            }
+
+            notifyListeners(this.jobContext, jobListener, TimingEvent.LauncherTimings.JOB_COMPLETE, new JobListenerAction() {
+              @Override
+              public void apply(JobListener jobListener, JobContext jobContext)
+                  throws Exception {
+                jobListener.onJobCompletion(jobContext);
+              }
+            });
+
+            if (jobState.getState() == JobState.RunningState.FAILED || jobState.getState() == JobState.RunningState.CANCELLED) {
+              notifyListeners(this.jobContext, jobListener, TimingEvent.LauncherTimings.JOB_FAILED, new JobListenerAction() {
+                @Override
+                public void apply(JobListener jobListener, JobContext jobContext)
+                    throws Exception {
+                  jobListener.onJobFailure(jobContext);
+                }
+              });
+              throw new JobException(String.format("Job %s failed", jobId));
+            } else {
+              notifyListeners(this.jobContext, jobListener, TimingEvent.LauncherTimings.JOB_SUCCEEDED, new JobListenerAction() {
+                @Override
+                public void apply(JobListener jobListener, JobContext jobContext)
+                    throws Exception {
+                  jobListener.onJobFailure(jobContext);
+                }
+              });
+            }
           }
         }
       }
-
-      for (JobState.DatasetState datasetState : this.jobContext.getDatasetStatesByUrns().values()) {
-        // Set the overall job state to FAILED if the job failed to process any dataset
-        if (datasetState.getState() == JobState.RunningState.FAILED) {
-          jobState.setState(JobState.RunningState.FAILED);
-          LOG.warn("At least one dataset state is FAILED. Setting job state to FAILED.");
-          break;
-        }
-      }
-
-      notifyListeners(this.jobContext, jobListener, TimingEvent.LauncherTimings.JOB_COMPLETE, new JobListenerAction() {
-        @Override
-        public void apply(JobListener jobListener, JobContext jobContext)
-            throws Exception {
-          jobListener.onJobCompletion(jobContext);
-        }
-      });
-
-      if (jobState.getState() == JobState.RunningState.FAILED) {
-        notifyListeners(this.jobContext, jobListener, TimingEvent.LauncherTimings.JOB_FAILED, new JobListenerAction() {
-          @Override
-          public void apply(JobListener jobListener, JobContext jobContext)
-              throws Exception {
-            jobListener.onJobFailure(jobContext);
-          }
-        });
-        throw new JobException(String.format("Job %s failed", jobId));
-      } else {
-        notifyListeners(this.jobContext, jobListener, TimingEvent.LauncherTimings.JOB_SUCCEEDED, new JobListenerAction() {
-          @Override
-          public void apply(JobListener jobListener, JobContext jobContext)
-              throws Exception {
-            jobListener.onJobFailure(jobContext);
-          }
-        });
-      }
     } finally {
       // Stop metrics reporting
       if (this.jobContext.getJobMetricsOptional().isPresent()) {
@@ -607,6 +660,13 @@ public abstract class AbstractJobLauncher implements JobLauncher {
     }
   }
 
+  @VisibleForTesting
+  public static long sumWorkUnitsSizes (WorkUnitStream workUnitStream) {
+    Collection<WorkUnit> workUnits = JobLauncherUtils.flattenWorkUnits(workUnitStream.getMaterializedWorkUnitCollection());
+    long totalSizeInBytes = workUnits.stream().mapToLong(wu -> wu.getPropAsLong(ServiceConfigKeys.WORK_UNIT_SIZE)).sum();
+    return totalSizeInBytes;
+  }
+
   private void executeUnfinishedCommitSequences(String jobName)
       throws IOException {
     Preconditions.checkState(this.jobContext.getCommitSequenceStore().isPresent());
@@ -643,6 +703,9 @@ public abstract class AbstractJobLauncher implements JobLauncher {
   @Override
   public void close()
       throws IOException {
+
+    troubleshooter.stop();
+
     try {
       this.cancellationExecutor.shutdownNow();
       try {
@@ -983,6 +1046,10 @@ public abstract class AbstractJobLauncher implements JobLauncher {
     return this.jobContext.getSource().isEarlyStopped();
   }
 
+  protected IssueRepository getIssueRepository() {
+    return troubleshooter.getIssueRepository();
+  }
+
   /**
    * Staging data cannot be cleaned if exactly once semantics is used, and the job has unfinished
    * commit sequences.
