@@ -18,32 +18,43 @@
 package org.apache.gobblin.runtime;
 
 import java.io.IOException;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.Properties;
 import java.util.Queue;
 import java.util.concurrent.Callable;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicBoolean;
 
-import lombok.Getter;
-import lombok.extern.slf4j.Slf4j;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.fs.Path;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import com.google.common.base.Optional;
+import com.google.common.base.Predicate;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Queues;
 import com.google.common.eventbus.EventBus;
-import com.google.common.util.concurrent.AbstractScheduledService;
-import com.google.common.base.Predicate;
 import com.google.common.io.Closer;
-import com.google.common.base.Optional;
+import com.google.common.util.concurrent.AbstractScheduledService;
+
+import lombok.Getter;
+import lombok.extern.slf4j.Slf4j;
 
 import org.apache.gobblin.configuration.ConfigurationKeys;
-import org.apache.gobblin.util.ClassAliasResolver;
-import org.apache.gobblin.util.ParallelRunner;
 import org.apache.gobblin.metastore.FsStateStore;
 import org.apache.gobblin.metastore.StateStore;
+import org.apache.gobblin.runtime.troubleshooter.Issue;
+import org.apache.gobblin.runtime.troubleshooter.IssueRepository;
+import org.apache.gobblin.runtime.troubleshooter.TroubleshooterException;
+import org.apache.gobblin.metrics.event.EventSubmitter;
+import org.apache.gobblin.metrics.event.TimingEvent;
+import org.apache.gobblin.service.ServiceConfigKeys;
+
+import org.apache.gobblin.util.ClassAliasResolver;
+import org.apache.gobblin.util.ParallelRunner;
 
 
 /**
@@ -63,6 +74,8 @@ public class TaskStateCollectorService extends AbstractScheduledService {
 
   private final EventBus eventBus;
 
+  private final EventSubmitter eventSubmitter;
+
   // Number of ParallelRunner threads to be used for state serialization/deserialization
   private final int stateSerDeRunnerThreads;
 
@@ -73,6 +86,12 @@ public class TaskStateCollectorService extends AbstractScheduledService {
 
   private final Path outputTaskStateDir;
 
+  private double totalSizeToCopy;
+
+  private double bytesCopiedSoFar;
+
+  private double lastPercentageReported;
+
   /**
    * Add a closeable action to run after each existence-checking of task state file.
    * A typical example to plug here is hive registration:
@@ -82,7 +101,7 @@ public class TaskStateCollectorService extends AbstractScheduledService {
   private final Optional<TaskStateCollectorServiceHandler> optionalTaskCollectorHandler;
   private final Closer handlerCloser = Closer.create();
 
-  private boolean isJobProceedOnCollectorServiceFailure;
+  private final boolean isJobProceedOnCollectorServiceFailure;
 
   /**
    * By default, whether {@link TaskStateCollectorService} finishes successfully or not won't influence
@@ -90,12 +109,18 @@ public class TaskStateCollectorService extends AbstractScheduledService {
    */
   private static final boolean defaultPolicyOnCollectorServiceFailure = true;
 
+  private final IssueRepository issueRepository;
+  private final AtomicBoolean reportedIssueConsumptionWarning = new AtomicBoolean(false);
+
   public TaskStateCollectorService(Properties jobProps, JobState jobState, EventBus eventBus,
-      StateStore<TaskState> taskStateStore, Path outputTaskStateDir) {
+      EventSubmitter eventSubmitter, StateStore<TaskState> taskStateStore, Path outputTaskStateDir,
+      IssueRepository issueRepository) {
     this.jobState = jobState;
     this.eventBus = eventBus;
+    this.eventSubmitter = eventSubmitter;
     this.taskStateStore = taskStateStore;
     this.outputTaskStateDir = outputTaskStateDir;
+    this.issueRepository = issueRepository;
 
     this.stateSerDeRunnerThreads = Integer.parseInt(jobProps.getProperty(ParallelRunner.PARALLEL_RUNNER_THREADS_KEY,
         Integer.toString(ParallelRunner.DEFAULT_PARALLEL_RUNNER_THREADS)));
@@ -200,8 +225,14 @@ public class TaskStateCollectorService extends AbstractScheduledService {
     // Add the TaskStates of completed tasks to the JobState so when the control
     // returns to the launcher, it sees the TaskStates of all completed tasks.
     for (TaskState taskState : taskStateQueue) {
+      consumeTaskIssues(taskState);
       taskState.setJobState(this.jobState);
       this.jobState.addTaskState(taskState);
+
+      if (this.jobState.getPropAsBoolean(ConfigurationKeys.REPORT_JOB_PROGRESS, ConfigurationKeys.DEFAULT_REPORT_JOB_PROGRESS)) {
+        LOGGER.info("Report job progress config is turned on");
+        reportJobProgress(taskState);
+      }
     }
 
     // Finish any additional steps defined in handler on driver level.
@@ -224,4 +255,62 @@ public class TaskStateCollectorService extends AbstractScheduledService {
     // Notify the listeners for the completion of the tasks
     this.eventBus.post(new NewTaskCompletionEvent(ImmutableList.copyOf(taskStateQueue)));
   }
+
+  /**
+   * Uses the size of work units to determine a job's progress and reports the progress as a percentage via
+   * GobblinTrackingEvents
+   * @param taskState of job launched
+   */
+  private void reportJobProgress(TaskState taskState) {
+    String stringSize = taskState.getProp(ServiceConfigKeys.WORK_UNIT_SIZE);
+    if (stringSize == null) {
+      LOGGER.warn("Expected to report job progress but work unit byte size property null");
+      return;
+    }
+
+    Long taskByteSize = Long.parseLong(stringSize);
+
+    // if progress reporting is enabled, value should be present
+    if (!this.jobState.contains(ServiceConfigKeys.TOTAL_WORK_UNIT_SIZE)) {
+      LOGGER.warn("Expected to report job progress but total bytes to copy property null");
+      return;
+    }
+    this.totalSizeToCopy = this.jobState.getPropAsLong(ServiceConfigKeys.TOTAL_WORK_UNIT_SIZE);
+
+    // avoid flooding Kafka message queue by sending GobblinTrackingEvents only when threshold is passed
+    this.bytesCopiedSoFar += taskByteSize;
+    Double newPercentageCopied = this.bytesCopiedSoFar / this.totalSizeToCopy;
+
+    if (newPercentageCopied - this.lastPercentageReported > ConfigurationKeys.DEFAULT_PROGRESS_REPORTING_THRESHOLD) {
+      this.lastPercentageReported = newPercentageCopied;
+      int percentageToReport = (int) Math.round(this.lastPercentageReported * 100);
+
+      Map<String, String> progress = new HashMap<>();
+      progress.put(TimingEvent.JOB_COMPLETION_PERCENTAGE, String.valueOf(percentageToReport));
+
+      LOGGER.info("Sending copy progress event with percentage " + percentageToReport + "%");
+      new TimingEvent(this.eventSubmitter, TimingEvent.JOB_COMPLETION_PERCENTAGE).stop(progress);
+    }
+  }
+
+  private void consumeTaskIssues(TaskState taskState) {
+    List<Issue> taskIssues = taskState.getTaskIssues();
+
+    /* A single job can spawn tens of thousands of tasks, and in case of wide-spread errors they will all produce
+     * similar Issues with large exception stack traces. If the process that collects task states keeps all of them in
+     * job state, it can run out of memory. To avoid that, we're forwarding issues to central repository that has
+     * size limits, and then remove them from the task state.
+     * */
+
+    if (taskIssues != null) {
+      try {
+        issueRepository.put(taskIssues);
+      } catch (TroubleshooterException e) {
+        if (reportedIssueConsumptionWarning.compareAndSet(false, true)) {
+          log.warn("Failed to consume task issues", e);
+        }
+      }
+      taskState.setTaskIssues(null);
+    }
+  }
 }
