@@ -21,7 +21,6 @@ import java.io.ByteArrayInputStream;
 import java.io.DataInputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.URI;
 import java.util.Collection;
 import java.util.List;
 
@@ -32,21 +31,18 @@ import org.apache.avro.io.Decoder;
 import org.apache.avro.io.DecoderFactory;
 import org.apache.avro.specific.SpecificDatumReader;
 
-import com.codahale.metrics.Meter;
-import com.google.common.base.Optional;
 import com.google.common.collect.Lists;
 import com.typesafe.config.Config;
 
-import org.apache.gobblin.kafka.client.GobblinKafkaConsumerClient;
+import lombok.Getter;
+import lombok.extern.slf4j.Slf4j;
+
+import org.apache.gobblin.metrics.ContextAwareMeter;
 import org.apache.gobblin.metrics.Tag;
 import org.apache.gobblin.metrics.reporter.util.SchemaVersionWriter;
 import org.apache.gobblin.runtime.api.JobSpec;
 import org.apache.gobblin.runtime.api.MutableJobCatalog;
 import org.apache.gobblin.runtime.metrics.RuntimeMetrics;
-import org.apache.gobblin.util.Either;
-
-import lombok.Getter;
-import lombok.extern.slf4j.Slf4j;
 
 
 /**
@@ -64,7 +60,7 @@ public abstract class KafkaAvroJobMonitor<T> extends KafkaJobMonitor {
   private final SchemaVersionWriter<?> versionWriter;
 
   @Getter
-  private Meter messageParseFailures;
+  private ContextAwareMeter messageParseFailures;
 
   public KafkaAvroJobMonitor(String topic, MutableJobCatalog catalog, Config config, Schema schema,
       SchemaVersionWriter<?> versionWriter) {
@@ -97,12 +93,12 @@ public abstract class KafkaAvroJobMonitor<T> extends KafkaJobMonitor {
   @Override
   protected void createMetrics() {
     super.createMetrics();
-    this.messageParseFailures = this.getMetricContext().meter(
+    this.messageParseFailures = this.getMetricContext().contextAwareMeter(
         RuntimeMetrics.GOBBLIN_JOB_MONITOR_KAFKA_MESSAGE_PARSE_FAILURES);
   }
 
   @Override
-  public Collection<Either<JobSpec, URI>> parseJobSpec(byte[] message)
+  public Collection<JobSpec> parseJobSpec(byte[] message)
       throws IOException {
 
     InputStream is = new ByteArrayInputStream(message);
@@ -126,5 +122,5 @@ public abstract class KafkaAvroJobMonitor<T> extends KafkaJobMonitor {
   /**
    * Extract {@link JobSpec}s from the Kafka message.
    */
-  public abstract Collection<Either<JobSpec, URI>> parseJobSpec(T message);
+  public abstract Collection<JobSpec> parseJobSpec(T message);
 }
