@@ -99,6 +99,8 @@ import org.apache.gobblin.runtime.TaskStateTracker;
 import org.apache.gobblin.runtime.job.GobblinJobFiniteStateMachine;
 import org.apache.gobblin.runtime.job.GobblinJobFiniteStateMachine.JobFSMState;
 import org.apache.gobblin.runtime.job.GobblinJobFiniteStateMachine.StateType;
+import org.apache.gobblin.runtime.troubleshooter.AutomaticTroubleshooter;
+import org.apache.gobblin.runtime.troubleshooter.AutomaticTroubleshooterFactory;
 import org.apache.gobblin.runtime.util.JobMetrics;
 import org.apache.gobblin.runtime.util.MetricGroup;
 import org.apache.gobblin.source.workunit.MultiWorkUnit;
@@ -248,8 +250,8 @@ public class MRJobLauncher extends AbstractJobLauncher {
     taskStateStore = new FsStateStore<>(this.fs, jobOutputPath.toString(), TaskState.class);
 
     this.taskStateCollectorService =
-        new TaskStateCollectorService(jobProps, this.jobContext.getJobState(), this.eventBus, taskStateStore,
-            outputTaskStateDir);
+        new TaskStateCollectorService(jobProps, this.jobContext.getJobState(), this.eventBus, this.eventSubmitter,
+            taskStateStore, outputTaskStateDir, getIssueRepository());
 
     this.jarFileMaximumRetry =
         jobProps.containsKey(ConfigurationKeys.MAXIMUM_JAR_COPY_RETRY_TIMES_KEY) ? Integer.parseInt(
@@ -735,11 +737,17 @@ public class MRJobLauncher extends AbstractJobLauncher {
     // A list of WorkUnits (flattened for MultiWorkUnits) to be run by this mapper
     private final List<WorkUnit> workUnits = Lists.newArrayList();
 
+    private AutomaticTroubleshooter troubleshooter;
+
     @Override
     protected void setup(Context context) {
       final State gobblinJobState = HadoopUtils.getStateFromConf(context.getConfiguration());
       TaskAttemptID taskAttemptID = context.getTaskAttemptID();
 
+      troubleshooter =
+          AutomaticTroubleshooterFactory.createForJob(ConfigUtils.propertiesToConfig(gobblinJobState.getProperties()));
+      troubleshooter.start();
+
       try (Closer closer = Closer.create()) {
         // Default for customizedProgressEnabled is false.
         this.customizedProgressEnabled = isCustomizedProgressReportEnabled(gobblinJobState.getProperties());
@@ -829,6 +837,8 @@ public class MRJobLauncher extends AbstractJobLauncher {
           }
         }
       }
+
+      AbstractJobLauncher.setDefaultAuthenticator(this.jobState.getProperties());
     }
 
     @Override
@@ -870,7 +880,7 @@ public class MRJobLauncher extends AbstractJobLauncher {
         gobblinMultiTaskAttempt =
             GobblinMultiTaskAttempt.runWorkUnits(this.jobState.getJobId(), context.getTaskAttemptID().toString(),
                 this.jobState, this.workUnits, this.taskStateTracker, this.taskExecutor, this.taskStateStore,
-                multiTaskAttemptCommitPolicy, jobBroker, (gmta) -> {
+                multiTaskAttemptCommitPolicy, jobBroker, troubleshooter.getIssueRepository(), (gmta) -> {
                   try {
                     return this.fs.exists(interruptPath);
                   } catch (IOException ioe) {
@@ -885,6 +895,14 @@ public class MRJobLauncher extends AbstractJobLauncher {
               .put(context.getTaskAttemptID().toString(), gobblinMultiTaskAttempt);
         }
       } finally {
+        try {
+          troubleshooter.refineIssues();
+          troubleshooter.logIssueSummary();
+          troubleshooter.stop();
+        } catch (Exception e) {
+          LOG.error("Failed to report issues from automatic troubleshooter", e);
+        }
+
         CommitStep cleanUpCommitStep = new CommitStep() {
 
           @Override
