@@ -20,19 +20,17 @@ package org.apache.gobblin.runtime.template;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.net.URI;
-import java.net.URISyntaxException;
 
 import com.google.common.base.Charsets;
 import com.typesafe.config.Config;
 import com.typesafe.config.ConfigFactory;
 
+import lombok.extern.slf4j.Slf4j;
+
 import org.apache.gobblin.configuration.ConfigurationKeys;
-import org.apache.gobblin.runtime.api.GobblinInstanceDriver;
 import org.apache.gobblin.runtime.api.JobCatalogWithTemplates;
 import org.apache.gobblin.runtime.api.SpecNotFoundException;
 
-import lombok.extern.slf4j.Slf4j;
-
 
 /**
  * A {@link org.apache.gobblin.runtime.api.JobTemplate} that loads a HOCON file as a {@link StaticJobTemplate}.
