@@ -71,6 +71,10 @@ public class DagManagerUtils {
     return jobSpec.getConfig().getLong(ConfigurationKeys.FLOW_EXECUTION_ID_KEY);
   }
 
+  static long getFlowExecId(String dagId) {
+    return Long.parseLong(dagId.substring(dagId.lastIndexOf('_') + 1));
+  }
+
   /**
    * Generate a dagId from the given {@link Dag} instance.
    * @param dag instance of a {@link Dag}.
@@ -96,18 +100,6 @@ public class DagManagerUtils {
     return Joiner.on("_").join(flowGroup, flowName, flowExecutionId);
   }
 
-  /**
-   * Generate a FlowId from the given {@link Dag} instance.
-   * FlowId, comparing to DagId, doesn't contain FlowExecutionId so different {@link Dag} could possibly have same
-   * {@link FlowId}.
-   * @param dag
-   * @return
-   */
-  static String generateFlowIdInString(Dag<JobExecutionPlan> dag) {
-    FlowId flowId = getFlowId(dag);
-    return Joiner.on("_").join(flowId.getFlowGroup(), flowId.getFlowName());
-  }
-
   /**
    * Returns a fully-qualified {@link Dag} name that includes: (flowGroup, flowName, flowExecutionId).
    * @param dag
@@ -185,7 +177,8 @@ public class DagManagerUtils {
       DagNode<JobExecutionPlan> node = nodesToExpand.poll();
       ExecutionStatus executionStatus = getExecutionStatus(node);
       boolean addFlag = true;
-      if (executionStatus == ExecutionStatus.PENDING || executionStatus == ExecutionStatus.PENDING_RETRY) {
+      if (executionStatus == ExecutionStatus.PENDING || executionStatus == ExecutionStatus.PENDING_RETRY
+          || executionStatus == ExecutionStatus.PENDING_RESUME) {
         //Add a node to be executed next, only if all of its parent nodes are COMPLETE.
         List<DagNode<JobExecutionPlan>> parentNodes = dag.getParents(node);
         for (DagNode<JobExecutionPlan> parentNode : parentNodes) {
@@ -243,12 +236,14 @@ public class DagManagerUtils {
   }
 
   /**
-   * flow start time is assumed to be same the flow execution id which is timestamp flow request was received
+   * Flow start time is the same as the flow execution id which is the timestamp flow request was received, unless it
+   * is a resumed flow, in which case it is {@link JobExecutionPlan#getFlowStartTime()}
    * @param dagNode dag node in context
-   * @return flow execution id
+   * @return flow start time
    */
   static long getFlowStartTime(DagNode<JobExecutionPlan> dagNode) {
-    return getFlowExecId(dagNode);
+    long flowStartTime = dagNode.getValue().getFlowStartTime();
+    return flowStartTime == 0L ? getFlowExecId(dagNode) : flowStartTime;
   }
 
   /**
