@@ -21,10 +21,14 @@ import java.io.File;
 import java.io.IOException;
 import java.lang.reflect.Type;
 import java.net.URI;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.stream.Collectors;
 
+import org.apache.commons.lang3.StringUtils;
+
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Charsets;
 import com.google.common.collect.Lists;
@@ -97,7 +101,15 @@ public class FSDagStateStore implements DagStateStore {
    */
   @Override
   public synchronized void cleanUp(Dag<JobExecutionPlan> dag) {
-    String fileName = DagManagerUtils.generateDagId(dag) + DAG_FILE_EXTENSION;
+    cleanUp(DagManagerUtils.generateDagId(dag));
+  }
+
+  /**
+   * {@inheritDoc}
+   */
+  @Override
+  public synchronized void cleanUp(String dagId) {
+    String fileName = dagId + DAG_FILE_EXTENSION;
 
     //Delete the dag checkpoint file from the checkpoint directory
     File checkpointFile = new File(this.dagCheckpointDir, fileName);
@@ -120,6 +132,18 @@ public class FSDagStateStore implements DagStateStore {
     return runningDags;
   }
 
+  /**
+   * {@inheritDoc}
+   */
+  @Override
+  public Dag<JobExecutionPlan> getDag(String dagId) throws IOException {
+    File file = new File(this.dagCheckpointDir, dagId + DAG_FILE_EXTENSION);
+    if (!file.exists()) {
+      return null;
+    }
+    return getDag(file);
+  }
+
   /**
    * Return a {@link Dag} given a file name.
    * @param dagFile
@@ -131,6 +155,20 @@ public class FSDagStateStore implements DagStateStore {
     return deserializeDag(serializedDag);
   }
 
+  /**
+   * {@inheritDoc}
+   */
+  @Override
+  public Set<String> getDagIds() {
+    Set<String> dagIds = new HashSet<>();
+    File dagCheckpointFolder = new File(this.dagCheckpointDir);
+
+    for (File file : dagCheckpointFolder.listFiles((dir, name) -> name.endsWith(DAG_FILE_EXTENSION))) {
+      dagIds.add(StringUtils.removeEnd(file.getName(), DAG_FILE_EXTENSION));
+    }
+    return dagIds;
+  }
+
   /**
    * Serialize a {@link Dag<JobExecutionPlan>}.
    * @param dag A Dag parametrized by type {@link JobExecutionPlan}.
