@@ -39,6 +39,8 @@ import com.google.common.collect.Maps;
 import com.typesafe.config.Config;
 
 import javax.annotation.Nonnull;
+import javax.inject.Inject;
+import javax.inject.Singleton;
 import lombok.Getter;
 import lombok.Setter;
 
@@ -67,11 +69,8 @@ import org.apache.gobblin.service.modules.flow.SpecCompiler;
 import org.apache.gobblin.service.modules.flowgraph.Dag;
 import org.apache.gobblin.service.modules.spec.JobExecutionPlan;
 import org.apache.gobblin.service.monitoring.FlowStatusGenerator;
-import org.apache.gobblin.service.monitoring.FsJobStatusRetriever;
-import org.apache.gobblin.service.monitoring.JobStatusRetriever;
 import org.apache.gobblin.util.ClassAliasResolver;
 import org.apache.gobblin.util.ConfigUtils;
-import org.apache.gobblin.util.reflection.GobblinConstructorUtils;
 
 
 /**
@@ -79,8 +78,8 @@ import org.apache.gobblin.util.reflection.GobblinConstructorUtils;
  * to {@link TopologyCatalog} and updates {@link SpecCompiler} state.
  */
 @Alpha
+@Singleton
 public class Orchestrator implements SpecCatalogListener, Instrumentable {
-  private static final String JOB_STATUS_RETRIEVER_CLASS_KEY = "jobStatusRetriever.class";
 
   protected final Logger _log;
   protected final SpecCompiler specCompiler;
@@ -97,6 +96,7 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
   private Optional<Meter> flowOrchestrationFailedMeter;
   @Getter
   private Optional<Timer> flowOrchestrationTimer;
+  private Optional<Meter> skippedFlowsMeter;
   @Setter
   private FlowStatusGenerator flowStatusGenerator;
 
@@ -104,12 +104,15 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
 
   private Map<String, FlowCompiledState> flowGauges = Maps.newHashMap();
 
-  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog, Optional<DagManager> dagManager, Optional<Logger> log,
-      boolean instrumentationEnabled) {
+
+  public Orchestrator(Config config, FlowStatusGenerator flowStatusGenerator, Optional<TopologyCatalog> topologyCatalog,
+      Optional<DagManager> dagManager, Optional<Logger> log, boolean instrumentationEnabled) {
     _log = log.isPresent() ? log.get() : LoggerFactory.getLogger(getClass());
     this.aliasResolver = new ClassAliasResolver<>(SpecCompiler.class);
     this.topologyCatalog = topologyCatalog;
     this.dagManager = dagManager;
+    this.flowStatusGenerator = flowStatusGenerator;
+
     try {
       String specCompilerClassName = ServiceConfigKeys.DEFAULT_GOBBLIN_SERVICE_FLOWCOMPILER_CLASS;
       if (config.hasPath(ServiceConfigKeys.GOBBLIN_SERVICE_FLOWCOMPILER_CLASS_KEY)) {
@@ -134,38 +137,26 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
       this.flowOrchestrationSuccessFulMeter = Optional.of(this.metricContext.meter(ServiceMetricNames.FLOW_ORCHESTRATION_SUCCESSFUL_METER));
       this.flowOrchestrationFailedMeter = Optional.of(this.metricContext.meter(ServiceMetricNames.FLOW_ORCHESTRATION_FAILED_METER));
       this.flowOrchestrationTimer = Optional.of(this.metricContext.timer(ServiceMetricNames.FLOW_ORCHESTRATION_TIMER));
+      this.skippedFlowsMeter = Optional.of(metricContext.contextAwareMeter(ServiceMetricNames.SKIPPED_FLOWS));
       this.eventSubmitter = Optional.of(new EventSubmitter.Builder(this.metricContext, "org.apache.gobblin.service").build());
     } else {
       this.metricContext = null;
       this.flowOrchestrationSuccessFulMeter = Optional.absent();
       this.flowOrchestrationFailedMeter = Optional.absent();
       this.flowOrchestrationTimer = Optional.absent();
+      this.skippedFlowsMeter = Optional.absent();
       this.eventSubmitter = Optional.absent();
     }
     this.flowConcurrencyFlag = ConfigUtils.getBoolean(config, ServiceConfigKeys.FLOW_CONCURRENCY_ALLOWED,
         ServiceConfigKeys.DEFAULT_FLOW_CONCURRENCY_ALLOWED);
   }
 
-  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog, Optional<DagManager> dagManager, Optional<Logger> log) {
-    this(config, topologyCatalog, dagManager, log, true);
+  @Inject
+  public Orchestrator(Config config, FlowStatusGenerator flowStatusGenerator, Optional<TopologyCatalog> topologyCatalog,
+      Optional<DagManager> dagManager, Optional<Logger> log) {
+    this(config, flowStatusGenerator, topologyCatalog, dagManager, log, true);
   }
 
-  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog, Optional<DagManager> dagManager, Logger log) {
-    this(config, topologyCatalog, dagManager, Optional.of(log));
-  }
-
-  public Orchestrator(Config config, Logger log) {
-    this(config, Optional.<TopologyCatalog>absent(), Optional.<DagManager>absent(), Optional.of(log));
-  }
-
-  /** Constructor with no logging */
-  public Orchestrator(Config config, Optional<TopologyCatalog> topologyCatalog) {
-    this(config, topologyCatalog, Optional.<DagManager>absent(), Optional.<Logger>absent());
-  }
-
-  public Orchestrator(Config config) {
-    this(config, Optional.<TopologyCatalog>absent(), Optional.<DagManager>absent(), Optional.<Logger>absent());
-  }
 
   @VisibleForTesting
   public SpecCompiler getSpecCompiler() {
@@ -247,6 +238,7 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
         _log.warn("Another instance of flowGroup: {}, flowName: {} running; Skipping flow execution since "
             + "concurrent executions are disabled for this flow.", flowGroup, flowName);
         flowGauges.get(spec.getUri().toString()).setState(CompiledState.SKIPPED);
+        Instrumented.markMeter(this.skippedFlowsMeter);
 
         // Send FLOW_FAILED event
         Map<String, String> flowMetadata = TimingEventUtils.getFlowMetadata((FlowSpec) spec);
@@ -302,7 +294,7 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
 
       if (this.dagManager.isPresent()) {
         //Send the dag to the DagManager.
-        this.dagManager.get().addDag(jobExecutionPlanDag, true);
+        this.dagManager.get().addDag(jobExecutionPlanDag, true, true);
       } else {
         // Schedule all compiled JobSpecs on their respective Executor
         for (Dag.DagNode<JobExecutionPlan> dagNode : jobExecutionPlanDag.getNodes()) {
@@ -365,51 +357,43 @@ public class Orchestrator implements SpecCatalogListener, Instrumentable {
     // .. this will work for Identity compiler but not always for multi-hop.
     // Note: Current logic assumes compilation is consistent between all executions
     if (spec instanceof FlowSpec) {
+      //Send the dag to the DagManager to stop it.
+      //Also send it to the SpecProducer to do any cleanup tasks on SpecExecutor.
       if (this.dagManager.isPresent()) {
-        //Send the dag to the DagManager.
         _log.info("Forwarding cancel request for flow URI {} to DagManager.", spec.getUri());
         this.dagManager.get().stopDag(spec.getUri());
-      } else {
-        // If DagManager is not enabled, we need to recompile the flow to find the spec producer,
-        // If compilation results is different, it remove request can go to some different spec producer
-        Dag<JobExecutionPlan> jobExecutionPlanDag = specCompiler.compileFlow(spec);
-
-        if (jobExecutionPlanDag.isEmpty()) {
-          _log.warn("Cannot determine an executor to delete Spec: " + spec);
-          return;
-        }
-
-        // Delete all compiled JobSpecs on their respective Executor
-        for (Dag.DagNode<JobExecutionPlan> dagNode: jobExecutionPlanDag.getNodes()) {
-          JobExecutionPlan jobExecutionPlan = dagNode.getValue();
-          Spec jobSpec = jobExecutionPlan.getJobSpec();
-          try {
-            SpecProducer<Spec> producer = jobExecutionPlan.getSpecExecutor().getProducer().get();
-            _log.info(String.format("Going to delete JobSpec: %s on Executor: %s", jobSpec, producer));
-            producer.deleteSpec(jobSpec.getUri(), headers);
-          } catch (Exception e) {
-            _log.error(String.format("Could not delete JobSpec: %s for flow: %s", jobSpec, spec), e);
-          }
-        }
       }
+      // We need to recompile the flow to find the spec producer,
+      // If compilation result is different, its remove request can go to some different spec producer
+      deleteFromExecutor(spec, headers);
     } else {
       throw new RuntimeException("Spec not of type FlowSpec, cannot delete: " + spec);
     }
   }
 
-  private FlowStatusGenerator buildFlowStatusGenerator(Config config) {
-    JobStatusRetriever jobStatusRetriever;
-    try {
-      Class jobStatusRetrieverClass = Class.forName(ConfigUtils.getString(config, JOB_STATUS_RETRIEVER_CLASS_KEY, FsJobStatusRetriever.class.getName()));
-      jobStatusRetriever =
-          (JobStatusRetriever) GobblinConstructorUtils.invokeLongestConstructor(jobStatusRetrieverClass, config);
-    } catch (ReflectiveOperationException e) {
-      _log.error("Exception encountered when instantiating JobStatusRetriever");
-      throw new RuntimeException(e);
+  private void deleteFromExecutor(Spec spec, Properties headers) {
+    Dag<JobExecutionPlan> jobExecutionPlanDag = specCompiler.compileFlow(spec);
+
+    if (jobExecutionPlanDag.isEmpty()) {
+      _log.warn("Cannot determine an executor to delete Spec: " + spec);
+      return;
+    }
+
+    // Delete all compiled JobSpecs on their respective Executor
+    for (Dag.DagNode<JobExecutionPlan> dagNode: jobExecutionPlanDag.getNodes()) {
+      JobExecutionPlan jobExecutionPlan = dagNode.getValue();
+      Spec jobSpec = jobExecutionPlan.getJobSpec();
+      try {
+        SpecProducer<Spec> producer = jobExecutionPlan.getSpecExecutor().getProducer().get();
+        _log.info(String.format("Going to delete JobSpec: %s on Executor: %s", jobSpec, producer));
+        producer.deleteSpec(jobSpec.getUri(), headers);
+      } catch (Exception e) {
+        _log.error(String.format("Could not delete JobSpec: %s for flow: %s", jobSpec, spec), e);
+      }
     }
-    return FlowStatusGenerator.builder().jobStatusRetriever(jobStatusRetriever).build();
   }
 
+
   @Nonnull
   @Override
   public MetricContext getMetricContext() {
