@@ -35,6 +35,8 @@ import com.linkedin.restli.server.CreateResponse;
 import com.linkedin.restli.server.UpdateResponse;
 import com.linkedin.restli.server.util.PatchApplier;
 
+import javax.inject.Inject;
+import javax.inject.Named;
 import lombok.Getter;
 import lombok.extern.slf4j.Slf4j;
 
@@ -48,6 +50,7 @@ import org.apache.gobblin.service.FlowId;
 import org.apache.gobblin.service.ServiceConfigKeys;
 import org.apache.gobblin.service.modules.scheduler.GobblinServiceJobScheduler;
 import org.apache.gobblin.service.modules.utils.HelixUtils;
+import org.apache.gobblin.service.modules.utils.InjectionNames;
 
 
 /**
@@ -63,16 +66,21 @@ public class GobblinServiceFlowConfigResourceHandler implements FlowConfigsResou
   private FlowConfigResourceLocalHandler localHandler;
   private Optional<HelixManager> helixManager;
   private GobblinServiceJobScheduler jobScheduler;
+  private boolean forceLeader;
 
-  public GobblinServiceFlowConfigResourceHandler(String serviceName, boolean flowCatalogLocalCommit,
+  @Inject
+  public GobblinServiceFlowConfigResourceHandler(@Named(InjectionNames.SERVICE_NAME) String serviceName,
+      @Named(InjectionNames.FLOW_CATALOG_LOCAL_COMMIT) boolean flowCatalogLocalCommit,
       FlowConfigResourceLocalHandler handler,
       Optional<HelixManager> manager,
-      GobblinServiceJobScheduler jobScheduler) {
+      GobblinServiceJobScheduler jobScheduler,
+      @Named(InjectionNames.FORCE_LEADER) boolean forceLeader) {
     this.flowCatalogLocalCommit = flowCatalogLocalCommit;
     this.serviceName = serviceName;
     this.localHandler = handler;
     this.helixManager = manager;
     this.jobScheduler = jobScheduler;
+    this.forceLeader = forceLeader;
   }
 
   @Override
@@ -117,6 +125,10 @@ public class GobblinServiceFlowConfigResourceHandler implements FlowConfigsResou
 
     checkHelixConnection(ServiceConfigKeys.HELIX_FLOWSPEC_ADD, flowName, flowGroup);
 
+    if (forceLeader) {
+      HelixUtils.throwErrorIfNotLeader(helixManager);
+    }
+
     try {
       if (!jobScheduler.isActive() && helixManager.isPresent()) {
         CreateResponse response = null;
@@ -167,6 +179,10 @@ public class GobblinServiceFlowConfigResourceHandler implements FlowConfigsResou
 
     checkHelixConnection(ServiceConfigKeys.HELIX_FLOWSPEC_UPDATE, flowName, flowGroup);
 
+    if (forceLeader) {
+      HelixUtils.throwErrorIfNotLeader(helixManager);
+    }
+
     try {
       if (!jobScheduler.isActive() && helixManager.isPresent()) {
 
@@ -223,6 +239,10 @@ public class GobblinServiceFlowConfigResourceHandler implements FlowConfigsResou
 
     checkHelixConnection(ServiceConfigKeys.HELIX_FLOWSPEC_REMOVE, flowName, flowGroup);
 
+    if (forceLeader) {
+      HelixUtils.throwErrorIfNotLeader(helixManager);
+    }
+
     try {
       if (!jobScheduler.isActive() && helixManager.isPresent()) {
 
