@@ -17,35 +17,30 @@
 
 package org.apache.gobblin.service.modules.scheduler;
 
-import java.io.IOException;
-import java.net.URI;
-import java.util.Arrays;
-import java.util.Iterator;
-import java.util.Map;
-import java.util.Properties;
-
-import org.apache.commons.lang.StringUtils;
-import org.apache.helix.HelixManager;
-import org.quartz.DisallowConcurrentExecution;
-import org.quartz.InterruptableJob;
-import org.quartz.JobDataMap;
-import org.quartz.JobExecutionContext;
-import org.quartz.JobExecutionException;
-import org.quartz.UnableToInterruptJobException;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Optional;
 import com.google.common.collect.Maps;
 import com.typesafe.config.Config;
 import com.typesafe.config.ConfigFactory;
-
+import java.io.IOException;
+import java.net.URI;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Properties;
+import javax.inject.Inject;
+import javax.inject.Named;
+import javax.inject.Singleton;
 import lombok.Getter;
 import lombok.extern.slf4j.Slf4j;
-
+import org.apache.commons.lang.StringUtils;
 import org.apache.gobblin.annotation.Alpha;
 import org.apache.gobblin.configuration.ConfigurationKeys;
+import org.apache.gobblin.instrumented.Instrumented;
+import org.apache.gobblin.metrics.ContextAwareMeter;
+import org.apache.gobblin.metrics.MetricContext;
+import org.apache.gobblin.metrics.ServiceMetricNames;
 import org.apache.gobblin.runtime.JobException;
 import org.apache.gobblin.runtime.api.FlowSpec;
 import org.apache.gobblin.runtime.api.Spec;
@@ -62,8 +57,19 @@ import org.apache.gobblin.service.modules.flowgraph.Dag;
 import org.apache.gobblin.service.modules.orchestration.DagManager;
 import org.apache.gobblin.service.modules.orchestration.Orchestrator;
 import org.apache.gobblin.service.modules.spec.JobExecutionPlan;
+import org.apache.gobblin.service.modules.utils.InjectionNames;
+import org.apache.gobblin.service.monitoring.FlowStatusGenerator;
 import org.apache.gobblin.util.ConfigUtils;
 import org.apache.gobblin.util.PropertiesUtils;
+import org.apache.helix.HelixManager;
+import org.quartz.DisallowConcurrentExecution;
+import org.quartz.InterruptableJob;
+import org.quartz.JobDataMap;
+import org.quartz.JobExecutionContext;
+import org.quartz.JobExecutionException;
+import org.quartz.UnableToInterruptJobException;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import static org.apache.gobblin.service.ServiceConfigKeys.GOBBLIN_SERVICE_PREFIX;
 
@@ -74,6 +80,7 @@ import static org.apache.gobblin.service.ServiceConfigKeys.GOBBLIN_SERVICE_PREFI
  * and runs them via {@link Orchestrator}.
  */
 @Alpha
+@Singleton
 public class GobblinServiceJobScheduler extends JobScheduler implements SpecCatalogListener {
 
   // Scheduler related configuration
@@ -90,6 +97,10 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
   @Getter
   private volatile boolean isActive;
   private String serviceName;
+  private static final MetricContext metricContext = Instrumented.getMetricContext(new org.apache.gobblin.configuration.State(),
+      GobblinServiceJobScheduler.class);
+  private static final ContextAwareMeter scheduledFlows = metricContext.contextAwareMeter(ServiceMetricNames.SCHEDULED_FLOW_METER);
+  private static final ContextAwareMeter nonScheduledFlows = metricContext.contextAwareMeter(ServiceMetricNames.NON_SCHEDULED_FLOW_METER);;
 
   /**
    * If current instances is nominated as a handler for DR traffic from down GaaS-Instance.
@@ -106,9 +117,10 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
    */
   public static final String DR_FILTER_TAG = "dr";
 
-  public GobblinServiceJobScheduler(String serviceName, Config config, Optional<HelixManager> helixManager,
-      Optional<FlowCatalog> flowCatalog, Optional<TopologyCatalog> topologyCatalog, Orchestrator orchestrator,
-      SchedulerService schedulerService, Optional<Logger> log) throws Exception {
+  @Inject
+  public GobblinServiceJobScheduler(@Named(InjectionNames.SERVICE_NAME) String serviceName, Config config,
+      Optional<HelixManager> helixManager, Optional<FlowCatalog> flowCatalog, Optional<TopologyCatalog> topologyCatalog,
+      Orchestrator orchestrator, SchedulerService schedulerService, Optional<Logger> log) throws Exception {
     super(ConfigUtils.configToProperties(config), schedulerService);
 
     _log = log.isPresent() ? log.get() : LoggerFactory.getLogger(getClass());
@@ -121,11 +133,12 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
         && config.hasPath(GOBBLIN_SERVICE_SCHEDULER_DR_NOMINATED);
   }
 
-  public GobblinServiceJobScheduler(String serviceName, Config config, Optional<HelixManager> helixManager,
+  public GobblinServiceJobScheduler(String serviceName, Config config, FlowStatusGenerator flowStatusGenerator,
+      Optional<HelixManager> helixManager,
       Optional<FlowCatalog> flowCatalog, Optional<TopologyCatalog> topologyCatalog, Optional<DagManager> dagManager,
-      SchedulerService schedulerService, Optional<Logger> log) throws Exception {
+      SchedulerService schedulerService,  Optional<Logger> log) throws Exception {
     this(serviceName, config, helixManager, flowCatalog, topologyCatalog,
-        new Orchestrator(config, topologyCatalog, dagManager, log), schedulerService, log);
+        new Orchestrator(config, flowStatusGenerator, topologyCatalog, dagManager, log), schedulerService, log);
   }
 
   public synchronized void setActive(boolean isActive) {
@@ -157,8 +170,13 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
       }
     } else {
       // Since we are going to change status to isActive=false, unschedule all flows
-      for (Spec spec : this.scheduledFlowSpecs.values()) {
-        onDeleteSpec(spec.getUri(), spec.getVersion());
+      List<Spec> specs = new ArrayList<>(this.scheduledFlowSpecs.values());
+      for (Spec spec : specs) {
+        try {
+          unscheduleSpec(spec.getUri(), spec.getVersion());
+        } catch (JobException e) {
+          _log.warn(String.format("Spec with URI: %s was not unscheduled during shutdown", spec.getUri()), e);
+        }
       }
       // Need to set active=false at the end; otherwise in the onDeleteSpec(), node will forward specs to active node, which is itself.
       this.isActive = isActive;
@@ -185,25 +203,27 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
         Iterator<URI> drUris = this.flowCatalog.get().getSpecURISWithTag(DR_FILTER_TAG);
         clearRunningFlowState(drUris);
       }
-
     } catch (IOException e) {
       throw new RuntimeException("Failed to get the iterator of all Spec URIS", e);
     }
 
-    try {
-      while (specUris.hasNext()) {
-        Spec spec = this.flowCatalog.get().getSpecWrapper(specUris.next());
-        //Disable FLOW_RUN_IMMEDIATELY on service startup or leadership change
-        if (spec instanceof FlowSpec) {
+    while (specUris.hasNext()) {
+      Spec spec = this.flowCatalog.get().getSpecWrapper(specUris.next());
+      try {
+        // Disable FLOW_RUN_IMMEDIATELY on service startup or leadership change if the property is set to true
+        if (spec instanceof FlowSpec && PropertiesUtils.getPropAsBoolean((
+            (FlowSpec) spec).getConfigAsProperties(), ConfigurationKeys.FLOW_RUN_IMMEDIATELY, "false")) {
           Spec modifiedSpec = disableFlowRunImmediatelyOnStart((FlowSpec) spec);
           onAddSpec(modifiedSpec);
         } else {
           onAddSpec(spec);
         }
+      } catch (Exception e) {
+        // If there is an uncaught error thrown during compilation, log it and continue adding flows
+        _log.error("Could not schedule spec {} from flowCatalog due to ", spec, e);
       }
-    } finally {
-      this.flowCatalog.get().getMetrics().updateGetSpecTime(startTime);
     }
+    this.flowCatalog.get().getMetrics().updateGetSpecTime(startTime);
   }
 
   /**
@@ -263,7 +283,7 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
   /**
    *
    * @param addedSpec spec to be added
-   * @return add spec response
+   * @return add spec response, which contains <code>null</code> if there is an error
    */
   @Override
   public AddSpecResponse onAddSpec(Spec addedSpec) {
@@ -288,16 +308,15 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
 
     // always try to compile the flow to verify if it is compilable
     Dag<JobExecutionPlan> dag = this.orchestrator.getSpecCompiler().compileFlow(flowSpec);
+    // If dag is null then a compilation error has occurred
     if (dag != null && !dag.isEmpty()) {
       response = dag.toString();
-    } else if (!flowSpec.getCompilationErrors().isEmpty()) {
-      response = Arrays.toString(flowSpec.getCompilationErrors().toArray());
     }
 
     boolean compileSuccess = FlowCatalog.isCompileSuccessful(response);
 
     if (isExplain || !compileSuccess || !this.isActive) {
-      // todo: in case of a scheudled job, we should also check if the job schedule is a valid cron schedule
+      // todo: in case of a scheduled job, we should also check if the job schedule is a valid cron schedule
       //  so it can be scheduled
       _log.info("Ignoring the spec {}. isExplain: {}, compileSuccess: {}, master: {}",
           addedSpec, isExplain, compileSuccess, this.isActive);
@@ -328,6 +347,25 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
     return new AddSpecResponse<>(response);
   }
 
+  /**
+   * Remove a flowSpec from schedule due to another leader being elected
+   * Unlike onDeleteSpec, we want to avoid deleting the flowSpec on the executor
+   * and we still want to unschedule if we cannot connect to zookeeper as the current node cannot be the master
+   * @param specURI
+   * @param specVersion
+   */
+  private void unscheduleSpec(URI specURI, String specVersion) throws JobException {
+    if (this.scheduledFlowSpecs.containsKey(specURI.toString())) {
+      _log.info("Unscheduling flowSpec " + specURI + "/" + specVersion);
+      this.scheduledFlowSpecs.remove(specURI.toString());
+      unscheduleJob(specURI.toString());
+    } else {
+      throw new JobException(String.format(
+          "Spec with URI: %s was not found in cache. May be it was cleaned, if not please clean it manually",
+          specURI));
+    }
+  }
+
   public void onDeleteSpec(URI deletedSpecURI, String deletedSpecVersion) {
     onDeleteSpec(deletedSpecURI, deletedSpecVersion, new Properties());
   }
@@ -345,15 +383,8 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
 
     try {
       Spec deletedSpec = this.scheduledFlowSpecs.get(deletedSpecURI.toString());
-      if (null != deletedSpec) {
-        this.orchestrator.remove(deletedSpec, headers);
-        this.scheduledFlowSpecs.remove(deletedSpecURI.toString());
-        unscheduleJob(deletedSpecURI.toString());
-      } else {
-        _log.warn(String.format(
-            "Spec with URI: %s was not found in cache. May be it was cleaned, if not please clean it manually",
-            deletedSpecURI));
-      }
+      unscheduleSpec(deletedSpecURI, deletedSpecVersion);
+      this.orchestrator.remove(deletedSpec, headers);
     } catch (JobException | IOException e) {
       _log.warn(String.format("Spec with URI: %s was not unscheduled cleaning", deletedSpecURI), e);
     }
@@ -375,11 +406,6 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
       return;
     }
 
-    try {
-      onDeleteSpec(updatedSpec.getUri(), updatedSpec.getVersion());
-    } catch (Exception e) {
-      _log.error("Failed to update Spec: " + updatedSpec, e);
-    }
     try {
       onAddSpec(updatedSpec);
     } catch (Exception e) {
@@ -429,6 +455,8 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
         jobScheduler.runJob(jobProps, jobListener);
       } catch (Throwable t) {
         throw new JobExecutionException(t);
+      } finally {
+        scheduledFlows.mark();
       }
     }
 
@@ -475,6 +503,8 @@ public class GobblinServiceJobScheduler extends JobScheduler implements SpecCata
         _log.error("Failed to run job " + this.jobConfig.getProperty(ConfigurationKeys.JOB_NAME_KEY), je);
       } catch (InterruptedException e) {
       _log.error("Failed to delete the spec " + specUri, e);
+      } finally {
+        nonScheduledFlows.mark();
       }
     }
   }
