@@ -54,6 +54,33 @@ public class PropertiesUtilsTest {
     Assert.assertTrue(!extractedPropertiesK3.containsKey("k2.kk"));
   }
 
+  @Test
+  public void testExtractPropertiesWithPrefixAfterRemovingPrefix() {
+
+    Properties properties = new Properties();
+    properties.setProperty("k1.kk1", "v1");
+    properties.setProperty("k1.kk2", "v2");
+    properties.setProperty("k2.kk", "v3");
+
+    // First prefix
+    Properties extractedPropertiesK1 = PropertiesUtils.extractPropertiesWithPrefixAfterRemovingPrefix(properties, "k1.");
+    Assert.assertEquals(extractedPropertiesK1.getProperty("kk1"), "v1");
+    Assert.assertEquals(extractedPropertiesK1.getProperty("kk2"), "v2");
+    Assert.assertTrue(!extractedPropertiesK1.containsKey("k2.kk"));
+
+    // Second prefix
+    Properties extractedPropertiesK2 = PropertiesUtils.extractPropertiesWithPrefixAfterRemovingPrefix(properties, "k2");
+    Assert.assertTrue(!extractedPropertiesK2.containsKey("k1.kk1"));
+    Assert.assertTrue(!extractedPropertiesK2.containsKey("k1.kk2"));
+    Assert.assertEquals(extractedPropertiesK2.getProperty(".kk"), "v3");
+
+    // Missing prefix
+    Properties extractedPropertiesK3 = PropertiesUtils.extractPropertiesWithPrefixAfterRemovingPrefix(properties, "k3");
+    Assert.assertTrue(!extractedPropertiesK3.containsKey("k1.kk1"));
+    Assert.assertTrue(!extractedPropertiesK3.containsKey("k1.kk1"));
+    Assert.assertTrue(!extractedPropertiesK3.containsKey("k2.kk"));
+  }
+
   @Test
   public void testGetStringList() {
     Properties properties = new Properties();
@@ -63,4 +90,15 @@ public class PropertiesUtilsTest {
     Assert.assertEquals(PropertiesUtils.getPropAsList(properties, "key"), ImmutableList.of("1", "2", "3"));
     Assert.assertEquals(PropertiesUtils.getPropAsList(properties, "key2", "default"), ImmutableList.of("default"));
   }
+
+  @Test
+  public void testGetValuesAsList() {
+    Properties properties = new Properties();
+    properties.put("k1", "v1");
+    properties.put("k2", "v2");
+    properties.put("k3", "v2");
+    properties.put("K3", "v4");
+
+    Assert.assertEqualsNoOrder(PropertiesUtils.getValuesAsList(properties, Optional.of("k")).toArray(), new String[]{"v1", "v2", "v2"});
+  }
 }
