@@ -19,11 +19,11 @@ package org.apache.gobblin.yarn;
 
 import java.io.File;
 import java.io.IOException;
-import java.lang.reflect.InvocationTargetException;
 import java.net.URI;
 import java.nio.ByteBuffer;
 import java.util.EnumSet;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
@@ -37,9 +37,8 @@ import java.util.concurrent.atomic.AtomicInteger;
 
 import org.apache.avro.Schema;
 import org.apache.commons.io.FileUtils;
-import org.apache.commons.lang3.StringUtils;
-import org.apache.commons.lang3.reflect.ConstructorUtils;
 import org.apache.commons.mail.EmailException;
+import org.apache.gobblin.util.hadoop.TokenUtils;
 import org.apache.gobblin.util.reflection.GobblinConstructorUtils;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileStatus;
@@ -47,9 +46,11 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.RawLocalFileSystem;
 import org.apache.hadoop.io.DataOutputBuffer;
+import org.apache.hadoop.io.Text;
 import org.apache.hadoop.security.Credentials;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.security.token.Token;
+import org.apache.hadoop.security.token.TokenIdentifier;
 import org.apache.hadoop.yarn.api.ApplicationConstants;
 import org.apache.hadoop.yarn.api.protocolrecords.GetNewApplicationResponse;
 import org.apache.hadoop.yarn.api.records.ApplicationAccessType;
@@ -194,7 +195,6 @@ public class GobblinYarnAppLauncher {
   private final HelixManager helixManager;
 
   private final Configuration yarnConfiguration;
-  private final YarnClient yarnClient;
   private final FileSystem fs;
 
   private final EventBus eventBus = new EventBus(GobblinYarnAppLauncher.class.getSimpleName());
@@ -238,6 +238,9 @@ public class GobblinYarnAppLauncher {
 
   private final String containerTimezone;
   private final String appLauncherMode;
+  private final String originalYarnRMAddress;
+  private final Map<String, YarnClient> potentialYarnClients;
+  private YarnClient yarnClient;
 
   public GobblinYarnAppLauncher(Config config, YarnConfiguration yarnConfiguration) throws IOException {
     this.config = config;
@@ -253,10 +256,19 @@ public class GobblinYarnAppLauncher {
         InstanceType.SPECTATOR, zkConnectionString);
 
     this.yarnConfiguration = yarnConfiguration;
+    YarnHelixUtils.setYarnClassPath(config, this.yarnConfiguration);
     YarnHelixUtils.setAdditionalYarnClassPath(config, this.yarnConfiguration);
     this.yarnConfiguration.set("fs.automatic.close", "false");
-    this.yarnClient = YarnClient.createYarnClient();
-    this.yarnClient.init(this.yarnConfiguration);
+    this.originalYarnRMAddress = this.yarnConfiguration.get(GobblinYarnConfigurationKeys.YARN_RESOURCE_MANAGER_ADDRESS);
+    this.potentialYarnClients = new HashMap();
+    Set<String> potentialRMAddresses = new HashSet<>(ConfigUtils.getStringList(config, GobblinYarnConfigurationKeys.OTHER_YARN_RESOURCE_MANAGER_ADDRESSES));
+    potentialRMAddresses.add(originalYarnRMAddress);
+    for (String rmAddress : potentialRMAddresses) {
+      YarnClient tmpYarnClient = YarnClient.createYarnClient();
+      this.yarnConfiguration.set(GobblinYarnConfigurationKeys.YARN_RESOURCE_MANAGER_ADDRESS, rmAddress);
+      tmpYarnClient.init(new YarnConfiguration(this.yarnConfiguration));
+      potentialYarnClients.put(rmAddress, tmpYarnClient);
+    }
 
     this.fs = GobblinClusterUtils.buildFileSystem(config, this.yarnConfiguration);
     this.closer.register(this.fs);
@@ -325,7 +337,7 @@ public class GobblinYarnAppLauncher {
    * @throws IOException if there's something wrong launching the application
    * @throws YarnException if there's something wrong launching the application
    */
-  public void launch() throws IOException, YarnException {
+  public void launch() throws IOException, YarnException, InterruptedException {
     this.eventBus.register(this);
 
     if (this.isHelixClusterManaged) {
@@ -342,20 +354,20 @@ public class GobblinYarnAppLauncher {
 
     connectHelixManager();
 
-    startYarnClient();
-
-    Optional<ApplicationId> reconnectableApplicationId = getReconnectableApplicationId();
-
-    boolean isReconnected = reconnectableApplicationId.isPresent();
-    // Before setup application, first login to make sure ugi has the right token.
+    //Before connect with yarn client, we need to login to get the token
     if(ConfigUtils.getBoolean(config, GobblinYarnConfigurationKeys.ENABLE_KEY_MANAGEMENT, false)) {
-      this.securityManager = Optional.of(buildSecurityManager(isReconnected));
+      this.securityManager = Optional.of(buildSecurityManager());
       this.securityManager.get().loginAndScheduleTokenRenewal();
     }
 
-    if (!reconnectableApplicationId.isPresent()) {
+    startYarnClient();
+
+    this.applicationId = getReconnectableApplicationId();
+
+    if (!this.applicationId.isPresent()) {
       disableLiveHelixInstances();
       LOGGER.info("No reconnectable application found so submitting a new application");
+      this.yarnClient = potentialYarnClients.get(this.originalYarnRMAddress);
       this.applicationId = Optional.of(setupAndSubmitApplication());
     }
 
@@ -551,12 +563,16 @@ public class GobblinYarnAppLauncher {
 
   @VisibleForTesting
   void startYarnClient() {
-    this.yarnClient.start();
+    for (YarnClient yarnClient : potentialYarnClients.values()) {
+      yarnClient.start();
+    }
   }
 
   @VisibleForTesting
   void stopYarnClient() {
-    this.yarnClient.stop();
+    for (YarnClient yarnClient : potentialYarnClients.values()) {
+      yarnClient.stop();
+    }
   }
 
   /**
@@ -576,19 +592,21 @@ public class GobblinYarnAppLauncher {
 
   @VisibleForTesting
   Optional<ApplicationId> getReconnectableApplicationId() throws YarnException, IOException {
-    List<ApplicationReport> applicationReports =
-        this.yarnClient.getApplications(APPLICATION_TYPES, RECONNECTABLE_APPLICATION_STATES);
-    if (applicationReports == null || applicationReports.isEmpty()) {
-      return Optional.absent();
-    }
+    for (YarnClient yarnClient: potentialYarnClients.values()) {
+      List<ApplicationReport> applicationReports = yarnClient.getApplications(APPLICATION_TYPES, RECONNECTABLE_APPLICATION_STATES);
+      if (applicationReports == null || applicationReports.isEmpty()) {
+        continue;
+      }
 
-    // Try to find an application with a matching application name
-    for (ApplicationReport applicationReport : applicationReports) {
-      if (this.applicationName.equals(applicationReport.getName())) {
-        String applicationId = sanitizeApplicationId(applicationReport.getApplicationId().toString());
-        LOGGER.info("Found reconnectable application with application ID: " + applicationId);
-        LOGGER.info("Application Tracking URL: " + applicationReport.getTrackingUrl());
-        return Optional.of(applicationReport.getApplicationId());
+      // Try to find an application with a matching application name
+      for (ApplicationReport applicationReport : applicationReports) {
+        if (this.applicationName.equals(applicationReport.getName())) {
+          String applicationId = sanitizeApplicationId(applicationReport.getApplicationId().toString());
+          LOGGER.info("Found reconnectable application with application ID: " + applicationId);
+          LOGGER.info("Application Tracking URL: " + applicationReport.getTrackingUrl());
+          this.yarnClient = yarnClient;
+          return Optional.of(applicationReport.getApplicationId());
+        }
       }
     }
 
@@ -602,12 +620,14 @@ public class GobblinYarnAppLauncher {
    * @throws YarnException if there's anything wrong setting up and submitting the Yarn application
    */
   @VisibleForTesting
-  ApplicationId setupAndSubmitApplication() throws IOException, YarnException {
+  ApplicationId setupAndSubmitApplication() throws IOException, YarnException, InterruptedException {
+    LOGGER.info("creating new yarn application");
     YarnClientApplication gobblinYarnApp = this.yarnClient.createApplication();
     ApplicationSubmissionContext appSubmissionContext = gobblinYarnApp.getApplicationSubmissionContext();
     appSubmissionContext.setApplicationType(GOBBLIN_YARN_APPLICATION_TYPE);
     appSubmissionContext.setMaxAppAttempts(ConfigUtils.getInt(config, GobblinYarnConfigurationKeys.APP_MASTER_MAX_ATTEMPTS_KEY, GobblinYarnConfigurationKeys.DEFAULT_APP_MASTER_MAX_ATTEMPTS_KEY));
     ApplicationId applicationId = appSubmissionContext.getApplicationId();
+    LOGGER.info("created new yarn application: "+ applicationId.getId());
 
     GetNewApplicationResponse newApplicationResponse = gobblinYarnApp.getNewApplicationResponse();
     // Set up resource type requirements for ApplicationMaster
@@ -697,8 +717,12 @@ public class GobblinYarnAppLauncher {
           Optional.of(appMasterResources), appFilesDestDir, localFs);
     }
     if (this.config.hasPath(GobblinYarnConfigurationKeys.APP_MASTER_FILES_REMOTE_KEY)) {
-      addAppRemoteFiles(this.config.getString(GobblinYarnConfigurationKeys.APP_MASTER_FILES_REMOTE_KEY),
-          appMasterResources);
+      YarnHelixUtils.addRemoteFilesToLocalResources(this.config.getString(GobblinYarnConfigurationKeys.APP_MASTER_FILES_REMOTE_KEY),
+          appMasterResources, yarnConfiguration);
+    }
+    if (this.config.hasPath(GobblinYarnConfigurationKeys.APP_MASTER_ZIPS_REMOTE_KEY)) {
+      YarnHelixUtils.addRemoteZipsToLocalResources(this.config.getString(GobblinYarnConfigurationKeys.APP_MASTER_ZIPS_REMOTE_KEY),
+          appMasterResources, yarnConfiguration);
     }
     if (this.config.hasPath(GobblinClusterConfigurationKeys.JOB_CONF_PATH_KEY)) {
       Path appFilesDestDir = new Path(appMasterWorkDir, GobblinYarnConfigurationKeys.APP_FILES_DIR_NAME);
@@ -784,13 +808,6 @@ public class GobblinYarnAppLauncher {
     }
   }
 
-  private void addAppRemoteFiles(String hdfsFileList, Map<String, LocalResource> resourceMap)
-      throws IOException {
-    for (String hdfsFilePath : SPLITTER.split(hdfsFileList)) {
-      YarnHelixUtils.addFileAsLocalResource(this.fs, new Path(hdfsFilePath), LocalResourceType.FILE, resourceMap);
-    }
-  }
-
   private void addJobConfPackage(String jobConfPackagePath, Path destDir, Map<String, LocalResource> resourceMap)
       throws IOException {
     Path srcFilePath = new Path(jobConfPackagePath);
@@ -821,36 +838,39 @@ public class GobblinYarnAppLauncher {
         .toString();
   }
 
-  private void setupSecurityTokens(ContainerLaunchContext containerLaunchContext) throws IOException {
+  private void setupSecurityTokens(ContainerLaunchContext containerLaunchContext) throws IOException, InterruptedException {
+    LOGGER.info("setting up SecurityTokens for containerLaunchContext.");
     Credentials credentials = UserGroupInformation.getCurrentUser().getCredentials();
+    String renewerName = this.yarnConfiguration.get(YarnConfiguration.RM_PRINCIPAL);
 
     // Pass on the credentials from the hadoop token file if present.
     // The value in the token file takes precedence.
     if (System.getenv(HADOOP_TOKEN_FILE_LOCATION) != null) {
+      LOGGER.info("HADOOP_TOKEN_FILE_LOCATION is set to {} reading tokens from it for containerLaunchContext.", System.getenv(HADOOP_TOKEN_FILE_LOCATION));
       Credentials tokenFileCredentials = Credentials.readTokenStorageFile(new File(System.getenv(HADOOP_TOKEN_FILE_LOCATION)),
-          new Configuration());
+              new Configuration());
       credentials.addAll(tokenFileCredentials);
+      LOGGER.debug("All containerLaunchContext tokens: {} present in file {} ", credentials.getAllTokens(), System.getenv(HADOOP_TOKEN_FILE_LOCATION));
     }
 
-    String tokenRenewer = this.yarnConfiguration.get(YarnConfiguration.RM_PRINCIPAL);
-    if (tokenRenewer == null || tokenRenewer.length() == 0) {
-      throw new IOException("Failed to get master Kerberos principal for the RM to use as renewer");
-    }
-
-    // For now, only getting tokens for the default file-system.
-    Token<?> tokens[] = this.fs.addDelegationTokens(tokenRenewer, credentials);
-    if (tokens != null) {
-      for (Token<?> token : tokens) {
-        LOGGER.info("Got delegation token for " + this.fs.getUri() + "; " + token);
+    TokenUtils.getAllFSTokens(new Configuration(), credentials, renewerName,
+        Optional.absent(), ConfigUtils.getStringList(this.config, TokenUtils.OTHER_NAMENODES));
+    // Only pass token here and no secrets. (since there is no simple way to remove single token/ get secrets)
+    // For RM token, only pass the RM token for the current RM, or the RM will fail to update the token
+    Credentials finalCredentials = new Credentials();
+    for (Token<? extends TokenIdentifier> token: credentials.getAllTokens()) {
+      if (token.getKind().equals(new Text("RM_DELEGATION_TOKEN")) && !token.getService().equals(new Text(this.originalYarnRMAddress))) {
+        continue;
       }
+      finalCredentials.addToken(token.getService(), token);
     }
-
     Closer closer = Closer.create();
     try {
       DataOutputBuffer dataOutputBuffer = closer.register(new DataOutputBuffer());
-      credentials.writeTokenStorageToStream(dataOutputBuffer);
+      finalCredentials.writeTokenStorageToStream(dataOutputBuffer);
       ByteBuffer fsTokens = ByteBuffer.wrap(dataOutputBuffer.getData(), 0, dataOutputBuffer.getLength());
       containerLaunchContext.setTokens(fsTokens);
+      LOGGER.info("Setting containerLaunchContext with All credential tokens: " + finalCredentials.getAllTokens());
     } catch (Throwable t) {
       throw closer.rethrow(t);
     } finally {
@@ -880,7 +900,7 @@ public class GobblinYarnAppLauncher {
     return logRootDir;
   }
 
-  private AbstractYarnAppSecurityManager buildSecurityManager(boolean isReconnected) throws IOException {
+  private AbstractYarnAppSecurityManager buildSecurityManager() throws IOException {
     Path tokenFilePath = new Path(this.fs.getHomeDirectory(), this.applicationName + Path.SEPARATOR +
         GobblinYarnConfigurationKeys.TOKEN_FILE_NAME);
 
@@ -889,7 +909,7 @@ public class GobblinYarnAppLauncher {
     try {
      return (AbstractYarnAppSecurityManager) GobblinConstructorUtils.invokeLongestConstructor(Class.forName(aliasResolver.resolve(
           ConfigUtils.getString(config, GobblinYarnConfigurationKeys.SECURITY_MANAGER_CLASS, GobblinYarnConfigurationKeys.DEFAULT_SECURITY_MANAGER_CLASS))), this.config, this.helixManager, this.fs,
-          tokenFilePath, isReconnected);
+          tokenFilePath);
     } catch (ReflectiveOperationException e) {
       throw new IOException(e);
     }
